var TargetedGT = function()
{
	var gt = {};
	var debug = 0;
	var manuallySelected = [];
	var selectionIsValid = false;
	var rawDataType;
	var pipeline;
	var annotationTypePipeline;
	var targets;
	var numTargetsSelected = 0;
	
	// Page initialization
	gt.initPage = function()
	{
		// Step 1
		Buttons.addClickHandler('btnSelectRawBioAssays', gt.selectRawBioAssays);
		Events.addEventHandler('rawBioAssays', 'base-selected', gt.setRawBioAssayCallback);
		Events.addEventHandler('rawBioAssays', 'change', gt.rawBioAssaysOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', gt.validateStep1);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', gt.initializeStep2);
		Events.addEventHandler('clusters', 'change', gt.clusterOnChange);
		Events.addEventHandler('gtSoftware', 'change', gt.softwareOnChange);
		Events.addEventHandler('step-2', 'wizard-validate', gt.validateStep2);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', gt.submit);

		var url = '../VariantCalling.servlet?ID='+App.getSessionId();
		url += '&cmd=GetTargetedGenotypeDefs';
		Wizard.asyncJsonRequest(url, gt.targetsLoaded);
	}
	
	gt.targetsLoaded = function(response)
	{
		targets = response.targets;
		if (targets.length == 0)
		{
			Wizard.setFatalError('No targeted genotype definitions have been configured in reggie-config.xml.');
			return;
		}
		
		var html = '';
		for (var tNo = 0; tNo < targets.length; tNo++)
		{
			var target = targets[tNo];
			html += '<tr class="align-top" id="targetrow.'+tNo+'">';
			html += '<td class="subprompt"><label for="target.'+tNo+'">'+Strings.encodeTags(target.name)+'</label></td>';
			html += '<td style="width: 2em;"><input type="checkbox" name="target.'+tNo+'" id="target.'+tNo+'"></td>';
			html += '<td class="input"><div class="targetdescription">'+(Strings.encodeTags(target.description) || '&nbsp;')+'</div></td>';
			html += '<td class="status" id="target.'+tNo+'.status"></td>';
			html += '<td class="help"><span id="target.'+tNo+'.message" class="message"></span></td>';
			html += '</tr>';
		}
		Doc.element('targets').innerHTML = html;
		Forms.linkCheckboxesWithLabels(Doc.element('targets'));
		
		for (var tNo = 0; tNo < targets.length; tNo++)
		{
			var target = targets[tNo];
			var checkbox = Doc.element('target.'+tNo);
			if (target.error)
			{
				Wizard.setInputStatus('target.'+tNo, 'warning', Reggie.softLineBreaks(target.error));
				checkbox.disabled = true;
			}
			else
			{
				checkbox.checked = true;
			}
			Events.addEventHandler(checkbox, 'change', gt.onTargetSelected);
		}
		
		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../VariantCalling.servlet?ID='+App.getSessionId();
		url += '&cmd=GetRawBioAssaysForTargetedGenotyping';
		Wizard.asyncJsonRequest(url, gt.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	gt.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];
		
		rawDataType = response.rawDataType;
		pipeline = response.pipeline;
		
		var rawBioAssays = response.rawBioAssays;
		if (rawBioAssays.length > 0)
		{
			var firstPlate = null;
			for (var rawNo=0; rawNo < rawBioAssays.length; rawNo++)
			{
				var raw = rawBioAssays[rawNo];
				
				var libPlate = raw.lib.bioWell ? raw.lib.bioWell.bioPlate : null;
				if (firstPlate == null && libPlate != null) firstPlate = libPlate.name;
				var option = gt.createListOption(rawNo+1, raw, raw.DO_NOT_USE == null && libPlate != null && libPlate.name==firstPlate)
				frm.rawBioAssays.options[frm.rawBioAssays.length] = option;
			}
			Events.sendChangeEvent('rawBioAssays');
		}
		else
		{
			Wizard.setInputStatus('rawBioAssays', 'invalid', 'No raw bioassays available for processing');
		}

		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	gt.validateStep1 = function(event)
	{
		if (!selectionIsValid) event.preventDefault();
	}

	gt.selectRawBioAssays = function()
	{
		var frm = document.forms['reggie'];
		if (frm.rawBioAssays.disabled) return;
		// Reset list
		manuallySelected = [];
		var url = '&resetTemporary=1';
		if (rawDataType) url += '&tmpfilter:STRING:rawDataType='+encodeURIComponent(rawDataType);
		if (pipeline != null)
		{
			if (annotationTypePipeline == null)
			{
				annotationTypePipeline = Reggie.getAnnotationTypeInfo('PIPELINE');
			}
			url += '&tmpfilter:STRING:'+encodeURIComponent('#')+annotationTypePipeline.id+'='+encodeURIComponent(pipeline);
		}
		Dialogs.selectItem('RAWBIOASSAY', 'rawBioAssays', 1, url);
	}

	gt.setRawBioAssayCallback = function(event)
	{
		var raw = event.detail;
		
		var opt = Reggie.getListOption('rawBioAssays', raw.id);
		if (opt)
		{
			opt.selected = true;
		}
		else
		{
			manuallySelected[manuallySelected.length] = raw.id;
		}
		
		if (raw.remaining == 0 && manuallySelected.length > 0)
		{
			var url = '../VariantCalling.servlet?ID='+App.getSessionId();
			url += '&cmd=GetRawBioAssaysForTargetedGenotyping';
			url += '&items='+manuallySelected.join(',');
			Wizard.showLoadingAnimation('Loading bioassays...');
			Wizard.asyncJsonRequest(url, gt.manuallySelected);
		}
		else
		{
			Events.sendChangeEvent('rawBioAssays');
		}
	}

	gt.manuallySelected = function(response)
	{
		var rawBioAssays = response.rawBioAssays;
		var frm = document.forms['reggie'];	

		if (rawBioAssays != null && rawBioAssays.length > 0)
		{
			var offset = frm.rawBioAssays.length+1;
			for (var rawNo=0; rawNo < rawBioAssays.length; rawNo++)
			{
				var raw = rawBioAssays[rawNo];
				var option = gt.createListOption(rawNo+offset, raw, raw.DO_NOT_USE == null);
				frm.rawBioAssays.options[frm.rawBioAssays.length] = option;
			}
			Events.sendChangeEvent('rawBioAssays');
		}
	}
	
	gt.createListOption = function(index, rawItem, selected)
	{
		var lib = rawItem.lib;
		var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
		var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
		
		var tooltip = null;
		var name = (index) + ': ';
		if (libPlate) name += libPlate.name + ' - ';
		name += rawItem.name;
		
		if (rawItem.DO_NOT_USE)
		{
			name += ' [DoNotUse]';
			tooltip = 'DoNotUse-'+Strings.encodeTags(rawItem.DO_NOT_USE+': '+rawItem.DO_NOT_USE_COMMENT);
		}
		else if (rawItem.AutoProcess == 'ReProcess')
		{
			name += ' [R]';
		}
		
		var option = new Option(name, rawItem.id, false, selected);
		if (isYellow) option.className = 'yellow';
		if (tooltip) option.title = tooltip;
		option.rawBioAssay = rawItem;
		return option;
	}

	gt.rawBioAssaysOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		selectionIsValid = false;
		var numSelected = 0;
		var numDoNotUse = 0;
		var invalidPipeline = null;

		Wizard.setInputStatus('rawBioAssays');
		Wizard.hideGoNextConfirmation();

		for (var rawNo = 0; rawNo < frm.rawBioAssays.length; rawNo++)
		{
			if (frm.rawBioAssays[rawNo].selected) 
			{
				numSelected++;
				
				var raw = frm.rawBioAssays[rawNo].rawBioAssay;
				if (raw.DO_NOT_USE) numDoNotUse++;
				
				if (pipeline && raw.pipeline && raw.pipeline != pipeline)
				{
					invalidPipeline = Strings.encodeTags(raw.name+' ('+raw.pipeline)+') is not intended for the '+pipeline+' pipeline.';
				}
			}
		}
		if (numSelected == 0)
		{
			Wizard.setInputStatus('rawBioAssays', 'invalid', 'Select at least one item');
			return;
		}
		if (invalidPipeline) 
		{
			Wizard.setInputStatus('rawBioAssays', 'invalid', invalidPipeline);
			return;
		}
		selectionIsValid = true;
		
		if (numDoNotUse > 0)
		{
			Wizard.setInputStatus('rawBioAssays', 'warning', numDoNotUse + ' selected items marked as DoNotUse');
			Wizard.showGoNextConfirmation(true, 'Confirm ' + numDoNotUse + ' items marked as DoNotUse');
		}
		else
		{
			Wizard.setInputStatus('rawBioAssays', 'valid');
		}
	}
	
	gt.initializeStep2 = function()
	{
		// Check debug by default if debug is set or not on a secure server (=production server)
		var frm = document.forms['reggie'];
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadSoftware('VARIANT_CALLING_SOFTWARE', 'gtSoftware', 'VARIANT_CALL_TYPE,PARAMETER_SET', 'TargetedGenotype');

		// Load open grid clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, gt.clustersLoaded);

		gt.onTargetSelected();
		Wizard.setCurrentStep(2);
		Doc.show('goregister');
		Doc.show('gocancel');
	}

	gt.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.hosts;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
			Wizard.setInputStatus('clusters', 'valid');
			clusterIsValid = true;
		}
		if (frm.clusters.length == 0)
		{
			Wizard.setInputStatus('clusters', 'invalid', 'No available Open Grid clusters');
			clusterIsValid = false;
		}
		else
		{
			gt.clusterOnChange();
		}
	}

	gt.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
	}

	gt.softwareOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var target = event.currentTarget;
		var item = target[target.selectedIndex].item;
		if (!item) item = {};
		
		Doc.element(target.id+'.parameterSet').innerHTML = Strings.encodeTags(item.ParameterSet || 'default');
		Doc.element(target.id+'.description').innerHTML = Strings.encodeTags(item.description);
		Wizard.setInputStatus(target.id, 'valid');

		if (item.ParameterSet)
		{
			var url = '../Install.servlet?ID='+App.getSessionId();
			url += '&cmd=GetParameterSetInfo';
			url += '&parameterSet='+encodeURIComponent(item.ParameterSet);
			url += '&targetId='+target.id;
			Wizard.asyncJsonRequest(url, gt.parameterSetInfoLoaded);
		}
	}
	
	gt.parameterSetInfoLoaded = function(response)
	{
		if (!response.parameters || response.parameters.length == 0)
		{
			Wizard.setInputStatus(response.targetId, 'warning', 'Can\'t find \'' + Strings.encodeTags(response.parameterSet) + '\' parameter set in reggie-config.xml');
		}
	}
	
	gt.onTargetSelected = function()
	{
		var frm = document.forms['reggie'];
		numTargetsSelected = 0;
		for (var tNo = 0; tNo < targets.length; tNo++)
		{
			if (frm['target.'+tNo].checked) numTargetsSelected++;
			Doc.addOrRemoveClass('targetrow.'+tNo, 'selected', frm['target.'+tNo].checked);
		}
		
		if (numTargetsSelected > 0)
		{
			Wizard.setInputStatus('targets', 'valid');
			Doc.removeClass('targets', 'invalid');
		}
		else
		{
			Wizard.setInputStatus('targets', 'invalid', 'Select at least one target!');
			Doc.addClass('targets', 'invalid');
		}
	}
	
	gt.validateStep2 = function(event)
	{
		if (numTargetsSelected == 0) event.preventDefault();
	}
	
	gt.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		
		var selectedRba = [];
		var selectedTargets = [];
		submitInfo.rawBioAssays = selectedRba;
		submitInfo.targets = selectedTargets;
		submitInfo.gtSoftware = parseInt(frm.gtSoftware.value);
		submitInfo.cluster = frm.clusters.value;
		if (frm.priority.selectedIndex >= 0)
		{
			submitInfo.priority = parseInt(frm.priority.value);
		}
		submitInfo.debug = frm.debug.checked;
		submitInfo.autoConfirm = frm.autoConfirm.checked;
		
		for (var rawNo = 0; rawNo < frm.rawBioAssays.length; rawNo++)
		{
			if (frm.rawBioAssays[rawNo].selected) 
			{
				selectedRba[selectedRba.length] = frm.rawBioAssays[rawNo].rawBioAssay.id;
			}
		}
		for (var tNo = 0; tNo < targets.length; tNo++)
		{
			if (frm['target.'+tNo].checked)
			{
				selectedTargets[selectedTargets.length] = targets[tNo].name;
			}
		}

		var url = '../VariantCalling.servlet?ID='+App.getSessionId();
		url += '&cmd=StartTargetedGenotypeAnalysis';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, gt.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	gt.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	return gt;
}();

Doc.onLoad(TargetedGT.initPage);

