var VCall = function()
{
	var vcall = {};
	var debug = 0;
	var selectionIsValid = false;
	var subtypeAlignedSequences = null;
	
	var clusterIsValid = false;
	var manuallySelected = [];
	
	// Page initialization
	vcall.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnSelectList', vcall.selectList);
		Events.addEventHandler('itemList', 'base-selected', Items.onItemSelected);
		Events.addEventHandler('itemList', 'change', vcall.itemListOnChange);
		Events.doOnEnter('itemList', vcall.selectList);
		Events.addEventHandler('step-1', 'wizard-validate', vcall.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', vcall.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', vcall.validateStep2);
		Events.addEventHandler('clusters', 'change', vcall.clusterOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', vcall.submit);

		Doc.show('step-1');
		Doc.show('gonext');
		
		var frm = document.forms['reggie'];
		frm.itemList.focus();
	}
	
	// --- Step 1 -----------------------------------
	
	vcall.validateStep1 = function(event)
	{
		if (!selectionIsValid) 
		{
			Wizard.setInputStatus('itemList', 'invalid', 'A list must be selected');
			event.preventDefault();
		}
	}
	
	vcall.selectList = function()
	{
		var frm = document.forms['reggie'];
		
		if (subtypeAlignedSequences == null)
		{
			subtypeAlignedSequences = Reggie.getSubtypeInfo('ALIGNED_SEQUENCES');
		}
		
		// Reset list
		manuallySelected = [];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:memberType=268'; // 268=DERIVEDBIOASSAY
		url += '&tmpfilter:INT:itemSubtype='+subtypeAlignedSequences.id;
		Dialogs.selectItem('ITEMLIST', 'itemList', 0, url);
	}

	vcall.itemListOnChange = function()
	{
		var frm = document.forms['reggie'];
		selectionIsValid = false;

		if (!frm.itemList.value)
		{
			Wizard.setInputStatus('itemList', 'invalid', 'A list must be selected');
			return;
		}

		// Make some minor and quick checks on the selected list
		Wizard.setInputStatus('itemList', 'checking', 'Checking...');
		var url = '../VariantCalling.servlet?ID='+App.getSessionId();
		url += '&cmd=PreValidateItemListForStatistics';
		url += '&listId='+frm.itemList.value;
		Wizard.asyncJsonRequest(url, vcall.itemListValidated);
	}

	vcall.itemListValidated = function(response)
	{
		if (response.message)
		{
			selectionIsValid = false;
			Wizard.setInputStatus('itemList', 'invalid', response.message);
		}
		else
		{
			selectionIsValid = true;
			Wizard.setInputStatus('itemList', 'valid');
		}
		
	}

	// --- Step 2 -----------------------------------
	vcall.initializeStep2 = function()
	{
		Wizard.setCurrentStep(2);
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		var frm = document.forms['reggie'];
		frm.debug.checked = debug || location.protocol != 'https:';

		// Load open grid clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, vcall.clustersLoaded);

		Doc.show('goregister');
		Doc.show('gocancel');
	}
	
	vcall.validateStep2 = function(event)
	{
		if (!clusterIsValid) event.preventDefault();
	}
	
	vcall.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.hosts;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
			Wizard.setInputStatus('clusters', 'valid');
			clusterIsValid = true;
		}
		if (frm.clusters.length == 0)
		{
			Wizard.setInputStatus('clusters', 'invalid', 'No available Open Grid clusters');
			clusterIsValid = false;
		}
		else
		{
			vcall.clusterOnChange();
		}
	}
	
	vcall.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
	}

	vcall.submit = function()
	{
		var frm = document.forms['reggie'];
			
		var submitInfo = {};
		submitInfo.itemList = parseInt(frm.itemList.value);
		submitInfo.cluster = frm.clusters.value;
		submitInfo.sampleType = frm.sampleType.value;
		if (frm.priority.selectedIndex >= 0)
		{
			submitInfo.priority = parseInt(frm.priority.value);
		}
		submitInfo.debug = frm.debug.checked;

		var url = '../VariantCalling.servlet?ID='+App.getSessionId();
		url += '&cmd=BuildVariantStatistics';

		Wizard.showLoadingAnimation('Performing registration...', 'variant-statistics-progress');
		Wizard.asyncJsonRequest(url, vcall.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	vcall.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return vcall;
}();

Doc.onLoad(VCall.initPage);

