var Search = function()
{
	var search = {};
	var debug = 0;
	
	var bugIcons = [ 'bug-yellow.png', 'bug-red.png', 'bomb.png' ];
	
	var alignments;
	var genotypecalls;
	var firstLoad = true;
	var numChangedFormElements = 0;
	
	var itemByName = {}; // To keep track of which QiaCubes and LibPlates we have already seen
	var assaysByName = {}; // Maps alignment/genotypecall name -> alignment/genotypecall object
	var alignmentsByPat = {}; // Maps patient name -> [] of alignments for that patient
	
	// Page initialization
	search.initPage = function()
	{
		// Step 1
		Events.addEventHandler('noFilter', 'change', search.filterOnChange);
		Events.addEventHandler('highHetFilter', 'change', search.filterOnChange);
		Events.addEventHandler('libPlateFilter', 'change', search.filterOnChange);
		Events.addEventHandler('qiaCubeFilter', 'change', search.filterOnChange);
		Events.addEventHandler('recentFilters', 'change', search.recentFilterOnChange);
		
		Events.addEventHandler('mode-genotype', 'change', search.modeOnChange);
		Events.addEventHandler('mode-reprocess', 'change', search.modeOnChange);
		Events.addEventHandler('mode-resolve', 'change', search.modeOnChange);
		
		Events.addEventHandler('itemTable', 'change', search.changedFormData);
		
		Buttons.addClickHandler('goreprocess', search.goReProcess);
		Buttons.addClickHandler('goresolve', search.goResolve);

		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=GetFlaggedAlignments';
		Wizard.showLoadingAnimation('Loading flagged alignments...');
		Wizard.asyncJsonRequest(url, search.onAlignmentsLoaded);
	}
	
	search.onAlignmentsLoaded = function(response)
	{
		alignments = response.alignments;
		genotypecalls = response.genotypecalls;
		
		for (var itemNo = 0; itemNo < genotypecalls.length; itemNo++)
		{
			var gtc = genotypecalls[itemNo];
			assaysByName[gtc.name] = gtc;
		}

		if (firstLoad) 
		{
			if (alignments.length == 0)
			{
				Wizard.setFatalError('No flagged alignments available for processing.');
				return;
			}
			search.initFilterLists();
		}
		else 
		{
			search.updateFilterLists();
		}
		firstLoad = false;
		search.primaryAnalysis();

		Doc.show('navigation');
		search.updateTable();
	}
	
	/**
		All changes to form elements will enable a warning when trying to 
		go somewhere else.
	*/
	search.changedFormData = function(event)
	{
		var ctrl = event.target;
		
		// Check if the control element has changed -- we need different checks for different element types
		var modified = false;
		if (ctrl.tagName == 'INPUT' && ctrl.type == 'text')
		{
			// For text elements we can simply compare the current value with the original value
			modified = ctrl.defaultValue != ctrl.value;
		}
		else if (ctrl.tagName == 'SELECT')
		{
			modified = ctrl.value != ''; // Works, since we always have no value for the default selection
		}
		else if (ctrl.tagName == 'INPUT' && ctrl.type == 'checkbox')
		{
			modified = ctrl.checked; // Works, since all checkboxes are unchecked by default
		}
		
		// Did the modified status change for the control?
		if (ctrl.modified != modified)
		{
			ctrl.modified = modified;
			// We need to apply the class to the parent (<td>) element since <select> elements don't support ::after/::before
			Doc.addOrRemoveClass(ctrl.parentNode, 'modified', modified);
			numChangedFormElements += modified ? 1 : -1; //Update counter for changed elements
			if (numChangedFormElements < 0) numChangedFormElements = 0; // Just to be safe...
			Wizard.setNoConfirmOnFirstStep(numChangedFormElements == 0);
		}
	}
	
	/**
		Initialize the QiaCube and LibPlate filter lists.
		This is done on the first load only.
	*/
	search.initFilterLists = function()
	{
		var allQiaCubes = [];
		var allLibPlates = [];
		
		for (var itemNo = 0; itemNo < alignments.length; itemNo++)
		{
			var aligned = alignments[itemNo];
			var rna = aligned.rna;
			if (rna)
			{
				var qiaCubeRun = rna.QIACUBE_DATE + '#' + rna.QIACUBE_RUN_NO;
				var qiaCubeRunPos = qiaCubeRun + '@' + rna.QIACUBE_POSITION;
		
				var qiaCube = itemByName[qiaCubeRun];
				if (!qiaCube)
				{
					qiaCube = new QiaCube(rna.QIACUBE_DATE, rna.QIACUBE_RUN_NO);
					qiaCube.posCount = 1;
					allQiaCubes[allQiaCubes.length] = qiaCube;
					itemByName[qiaCubeRun] = qiaCube;
					itemByName[qiaCubeRunPos] = qiaCube;
				}
				else
				{
					if (!itemByName[qiaCubeRunPos])
					{
						qiaCube.posCount++;
						itemByName[qiaCubeRunPos] = qiaCube;
					}
				}
			}
			
			var lib = aligned.lib;
			if (lib)
			{
				var libPlate = lib.bioWell.bioPlate;
		
				var plate = itemByName[libPlate.name];
				if (!plate)
				{
					plate = new LibPlate(libPlate.name);
					plate.posCount = 1;
					allLibPlates[allLibPlates.length] = plate;
					itemByName[libPlate.name] = plate;
					itemByName[platePos] = plate;
				}
				else
				{
					var platePos = libPlate.name+'@'+lib.bioWell.location;
					if (!itemByName[platePos])
					{
						plate.posCount++;
						itemByName[platePos] = plate;
					}
				}
			}
		}
		
		// Sort in logical order
		allQiaCubes.sort(Sort.sortByName);
		allLibPlates.sort(Sort.sortByName);
		
		var frm = document.forms['reggie'];
		for (var plateNo = 0; plateNo < allLibPlates.length; plateNo++)
		{
			var libPlate = allLibPlates[plateNo];
			var option = new Option(libPlate.name + " [" + libPlate.posCount + "]", libPlate.name);
			option.libPlate = libPlate;
			frm.libPlateFilter[frm.libPlateFilter.length] = option;
		}
		for (var qiaCubeNo = 0; qiaCubeNo < allQiaCubes.length; qiaCubeNo++)
		{
			var qiaCube = allQiaCubes[qiaCubeNo];
			var option = new Option(qiaCube.title + " [" + qiaCube.posCount + "]", qiaCube.name);
			option.qiaCube = qiaCube;
			frm.qiaCubeFilter[frm.qiaCubeFilter.length] = option;
		}
	}
	
	
	/**
		Update the QiaCube and LibPlate filter lists with new options.
		This is done after the first load. The options are added to the end of the lists.
	*/
	search.updateFilterLists = function()
	{
		var frm = document.forms['reggie'];
		
		for (var itemNo = 0; itemNo < alignments.length; itemNo++)
		{
			var aligned = alignments[itemNo];
			if (aligned.specimen)
			{
				var rna = aligned.rna;
				var qiaCubeRun = rna.QIACUBE_DATE + '#' + rna.QIACUBE_RUN_NO;
				var qiaCubeRunPos = qiaCubeRun + '@' + rna.QIACUBE_POSITION;
				var qiaCube = itemByName[qiaCubeRun];
				if (!qiaCube)
				{
					qiaCube = new QiaCube(rna.QIACUBE_DATE, rna.QIACUBE_RUN_NO);
					itemByName[qiaCubeRun] = qiaCube;
					var option = new Option(qiaCube.title, qiaCube.name);
					option.qiaCube = qiaCube;
					frm.qiaCubeFilter[frm.qiaCubeFilter.length] = option;
				}
			}
			
			var lib = aligned.lib;
			if (lib)
			{
				var libPlate = lib.bioWell.bioPlate;
				var plate = itemByName[libPlate.name];
				if (!plate)
				{
					plate = new LibPlate(libPlate.name);
					itemByName[libPlate.name] = plate;
					
					var option = new Option(libPlate.name, libPlate.name);
					option.libPlate = libPlate;
					frm.libPlateFilter[frm.libPlateFilter.length] = option;
				}
			}
		}
	}
	
	/**
		Callback when some of the filter options has changed.
		This method will detect which one was modified, and
		reset all other filters. Then, it will issue a 
		request to the server to load alignments related to the
		selected filter.
	*/
	search.filterOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		
		var noFilter = Doc.element('noFilter');
		var highHet = Doc.element('highHetFilter');
		var libPlate = Doc.element('libPlateFilter');
		var qiaCube = Doc.element('qiaCubeFilter');
		
		if (event)
		{
			if (event.target != noFilter) noFilter.checked = false;
			if (event.target != highHet) highHet.checked = false;
			if (event.target != libPlate) libPlate.selectedIndex = 0;
			if (event.target != qiaCube) qiaCube.selectedIndex = 0;
		}
		
		if (numChangedFormElements > 0)
		{
			if (!confirm('There are unsaved changes. Discard changes?')) return;
		}
		
		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=GetFlaggedAlignments';
		
		var msg = 'Loading flagged alignments...';
		var hasFilter = false;
		var newRecentFilter = null;
		if (libPlate.selectedIndex > 0)
		{
			url += '&libPlateFilter='+encodeURIComponent(libPlate.value);
			msg = 'Loading alignments related to ' + libPlate.value;
			hasFilter = true;
			newRecentFilter = { 'value': 'lp:'+libPlate.value, 'text': libPlate.value };
		}
		else if (qiaCube.selectedIndex > 0)
		{
			var qiaCube = frm.qiaCubeFilter[frm.qiaCubeFilter.selectedIndex].qiaCube;
			url += '&qiaCubeFilter='+encodeURIComponent(qiaCube.name);
			msg = 'Loading alignments related to QiaCube ' + qiaCube.title;
			hasFilter = true;
			newRecentFilter = { 'value': 'qc:'+qiaCube.name, 'text': qiaCube.title };
		}
		else if (highHet.checked)
		{
			url += '&highHetFilter=1';
			hasFilter = true;
		}
		else
		{
			noFilter.checked = true;
			Forms.checkRadio(frm.mode, 'genotype');
		}
		
		Wizard.hideGoNextConfirmation();
		Wizard.showLoadingAnimation(msg);
		Wizard.asyncJsonRequest(url, search.onAlignmentsLoaded);
		
		if (newRecentFilter != null)
		{
			var recentFilter = Doc.element('recentFilters');
			for (var i = 0; i < recentFilter.length; i++)
			{
				if (recentFilter[i].value == newRecentFilter.value)
				{
					recentFilter.remove(i);
					break;
				}
			}
			recentFilter.add(new Option(newRecentFilter.text, newRecentFilter.value), 1);
			Doc.show(recentFilter, 'inline');
		}
		
		// Hide buttons while updating and disable mode changes
		Doc.hide('navigation');
		Doc.element('mode-reprocess').disabled = true;
		Doc.element('mode-resolve').disabled = true;
		
		if (hasFilter && !highHet.checked)
		{
			search.updateTable(true);
		}
		else
		{
			Doc.element('itemTable').innerHTML = '';
		}
	}
	

	search.recentFilterOnChange = function(event)
	{
		var list = event.currentTarget;
		var value = list.value;
		var prefix = value.substr(0, 3);
		list.selectedIndex = 0;
		if (prefix == 'qc:')
		{
			Data.set(event.currentTarget, 'qiacuberun', value.substr(3));
			search.setQiaCubeFilter(event);
		}
		else if (prefix == 'lp:')
		{
			Data.set(event.currentTarget, 'libplate', value.substr(3));
			search.setLibPlateFilter(event);
		}
		
	}
	
	/**
		Event handler for clicking on a QiaCube run, which should
		enable the filter for that run and re-load alignments 
		related to it. Updating the selected item in the filter list
		and sending a 'change' event should do the trick.
	*/
	search.setQiaCubeFilter = function(event)
	{
		var qiaCubeRun = Data.get(event.currentTarget, 'qiacuberun');
		var frm = document.forms['reggie'];
		Forms.selectListOption(frm.qiaCubeFilter, qiaCubeRun);
		Events.sendChangeEvent(frm.qiaCubeFilter);
	}
	
	/**
		Event handler for clicking on a LibPlate, which should
		enable the filter for that run and re-load alignments 
		related to it. Updating the selected item in the filter list
		and sending a 'change' event should do the trick.
	*/
	search.setLibPlateFilter = function(event)
	{
		var libplate = Data.get(event.currentTarget, 'libplate');
		var frm = document.forms['reggie'];
		Forms.selectListOption(frm.libPlateFilter, libplate);
		Events.sendChangeEvent(frm.libPlateFilter);
	}

	
	search.updateTable = function(noContextAnalysis)
	{
		var frm = document.forms['reggie'];
		
		var stepTitle = 'Flagged alignments';
		var filter = null;
		var filterType = null;
		var positionProperty = null;
		var highHetFilter = false;
		var qiacubefilter = false;
		var libplatefilter = false;
		var qiaCube = null;
		var libPlate = null;
		
		if (Doc.element('highHetFilter').checked)
		{
			alignments.sort(Sort.sortAlignmentsByLibPlatePosition);
			highHetFilter = true;
			filter = Filter.hetFilter(65);
			filterType = 'HET';
			positionProperty = Property.LibPlatePosition;
			stepTitle = 'Alignments with HET &gt; 65%';
		}
		else if (frm.qiaCubeFilter.selectedIndex > 0)
		{
			alignments.sort(Sort.sortAlignmentsByQiaCubePosition);
			qiaCube = frm.qiaCubeFilter[frm.qiaCubeFilter.selectedIndex].qiaCube;
			qiacubefilter = qiaCube.name;
			filter = Filter.qiaCubeFilter(qiaCube.name);
			filterType = 'QIACUBE';
			positionProperty = Property.QiaCubePosition;
			stepTitle = 'Alignments related to QiaCube ' + qiaCube.title;
		}
		else if (frm.libPlateFilter.selectedIndex > 0)
		{
			alignments.sort(Sort.sortAlignmentsByLibPlatePosition);
			libPlate = frm.libPlateFilter[frm.libPlateFilter.selectedIndex].libPlate;
			libplatefilter = libPlate.name;
			filter = Filter.libPlateFilter(libPlate.name);
			filterType = 'LIBPLATE';
			positionProperty = Property.LibPlatePosition;
			stepTitle = 'Alignments related to ' + libPlate.name;
		}
		else
		{
			// No filter
			alignments.sort(Sort.sortAlignmentsByQiaCubePosition);
		}
		
		if (filter && positionProperty && !noContextAnalysis && !highHetFilter)
		{
			search.contextAnalysis(filter, positionProperty);
		}

		var filtered = [];
		var tbody = [];
		if (filter)
		{
			var lastPos = null;
			var lastStartTb = null;
			for (var itemNo = 0; itemNo < alignments.length; itemNo++)
			{
				var aligned = alignments[itemNo];
				if (filter(aligned)) 
				{
					filtered[filtered.length] = aligned;
					var tb = { 'start': 1, 'end': 1 }; // The default is to start a new <tbody> section for each alignment
					tb.rowspan = aligned.compare ? (aligned.compare.messages.length || 1) : 1; // A <tbody> typically spans over number-or-messages rows
					tb.tbodyspan = tb.rowspan;
					tbody[tbody.length] = tb;
					if (positionProperty)
					{
						var pos = positionProperty(aligned);
						if (pos == lastPos)
						{
							// If we are at the same position, we should not start a new <tbody> section
							// Adjust properties accordingly
							tbody[tbody.length-2].end = 0;
							tb.start = 0;
							lastStartTb.tbodyspan += tb.rowspan;
						}
						else
						{
							// New position in the context
							lastPos = pos;
							lastStartTb = tb;
						}
					}
				}
			}
		}
		else
		{
			filtered = alignments;
		}
		
		var html = '';
		
		if (filter)
		{
			html += '<thead class="mode-reprocess"><tr>';
			html += '<th colspan="4"></th>';
			html += '<th colspan="3" class="mode-reprocess-header">Remaining quantity</th>';
			html += '<th colspan="10"></th>';
			html += '</tr></thead>';
		}
		
		html += '<thead class="bg-filled-100">';
		html += '<tr>';
		html += '<th>Patient</th>';
		html += '<th class="'+(qiacubefilter?'':'mode-genotype')+'">QiaCube</th>';
		html += '<th class="'+(libplatefilter||highHetFilter?'':'mode-genotype')+'">LibPlate</th>';
		html += '<th>HET</th>';
		html += '<th>Alignment</th>';
		if (filter)
		{
			html += '<th class="mode-reprocess dottedleft num-col">Specimen<br>(mg)</th>';
			html += '<th class="mode-reprocess num-col">Lysate<br>(µl)</th>';
			html += '<th class="mode-reprocess num-col">RNA<br>(µg)</th>';
			html += '<th class="mode-reprocess" id="reprocess-header">Re-process</th>';
			// Note! "mode-resolve" columns here to get the same order for grouping
			html += '<th class="mode-resolve dottedleft">Ok</th>';
			html += '<th class="mode-resolve" colspan="2">DoNotUse<span style="float:right;">FutureGT</span></th>';
			html += '<th class="mode-resolve comment-col">Comment</th>';

			html += '<th class="mode-reprocess comment-col">Comment</th>';
			html += '<th class="mode-reprocess flag-col dottedright">Flag</th>';
		}
		html += '<th></th>';
		html += '<th class="dottedleft" colspan="2">Warnings and messages</th>';
		html += '<th class="dottedleft">Alignment/GenotypeCall</th>';
		html += '<th>HET</th>';
		html += '<th class="mode-genotype">QiaCube</th>';
		html += '<th class="mode-genotype">LibPlate/BeadChip</th>';
		html += '<th>Patient</th>';
		html += '</tr>';
		html += '</thead>';
		
		var numListed = 0;
		for (var itemNo = 0; itemNo < filtered.length; itemNo++)
		{
			var aligned = filtered[itemNo];
			var cmp = aligned.compare;
			var lib = aligned.lib;
			var rna = aligned.rna;
			var lys = aligned.lysate;
			var sp = aligned.specimen;
			var recommendReprocess = false;
			var recommendDoNotUse = false;
			
			var libPlate = lib ? lib.bioWell.bioPlate : null;
			numListed++;

			var tb = tbody[itemNo] || { 'start': 1, 'end': 1 };
			var rowspan = tb.rowspan || (cmp ? cmp.messages.length : 0) || 1;
			var tbodySpan = tb.tbodyspan || rowspan;
			
			if (!tb.start)
			{
				html += '<tr class="dotted-above">';
			}
			else
			{
				// Try to give a re-process recommendation based on the assumption that the
				// current alignment is incorrect. We look for other alignments from the same
				// patient and specimen but with either a different RNA or Library
				if (filter && aligned.flagged && !aligned.QC_GENOTYPE_VERIFIED && sp)
				{
					recommendReprocess = true; 
					var otherAlignments = alignmentsByPat[sp.patientName];
					for (var otherNo = 0; otherNo < otherAlignments.length; otherNo++)
					{
						var otherA = otherAlignments[otherNo];				
						if (otherA.bugged && otherA.buglevel > 0) continue; // Ignore items that we "know" to be incorrect
						
						var sameLib = otherA.lib && otherA.lib.id == lib.id;
						var sameRNA = otherA.rna && otherA.rna.id == rna.id;
						var sameSp = otherA.specimen && otherA.specimen.id == sp.id;
						if (sameSp)
						{
							if (!sameRNA && qiacubefilter || !sameLib && (libplatefilter || highHetFilter)) 
							{
								recommendReprocess = false;
							}
							if (!sameLib && highHetFilter) recommendDoNotUse = true;
						}
					}
				}
				
				html += '<tbody class="highlight'+(recommendReprocess ? ' recommend-reprocess' : '')+(recommendDoNotUse ? ' recommend-donotuse' : '')+'" id="section.'+aligned.tmpId+'"><tr>';
				html += '<td rowspan="'+tbodySpan+'">'+(sp ? HTMLHelper.colorMatch(sp.patientName, null, 'Click to highlight this patient') : '')+'</td>';
				html += '<td rowspan="'+tbodySpan+'" class="'+(qiacubefilter?'':'mode-genotype')+'">'+(sp ? HTMLHelper.qiaCubeLink(rna.QIACUBE_DATE, rna.QIACUBE_RUN_NO, rna.QIACUBE_POSITION, qiacubefilter, false) : '')+'</td>';
				if (!qiacubefilter)
				{
					html += '<td rowspan="'+tbodySpan+'">'+(libPlate ? HTMLHelper.libPlateLink(libPlate.name, lib.bioWell.location, libplatefilter, false):'')+'</td>';
				}
			}
			
			var tmp = search.getAlignmentHtml(aligned);
			
			if (qiacubefilter)
			{
				html += '<td rowspan="'+rowspan+'" class="mode-genotype">'+(libPlate ? HTMLHelper.libPlateLink(libPlate.name, lib.bioWell.location, libplatefilter, false):'')+'</td>';
			}
			html += '<td rowspan="'+rowspan+'">'+Reggie.formatNumber(aligned.QC_GENOTYPE_HET_PCT, '%', -1)+'</td>';
			html += '<td rowspan="'+rowspan+'" class="text-col">'+tmp.name+'</td>';
			if (filter)
			{
				if (tb.start)
				{
					html += '<td rowspan="'+tbodySpan+'" class="mode-reprocess dottedleft num-col">'+(sp ? Reggie.formatNumber(sp.remainingQuantity/1000, null, 1, 1) : '')+'</td>';
					html += '<td rowspan="'+tbodySpan+'" class="mode-reprocess num-col">'+(lys ? Math.round(lys.remainingQuantity) : '')+'</td>';
					html += '<td rowspan="'+tbodySpan+'" class="mode-reprocess num-col">'+(sp ? Reggie.formatNumber(rna.remainingQuantity, null, 2, 2) : '')+'</td>';
					html += '<td rowspan="'+tbodySpan+'" class="mode-reprocess reprocess-col editable">'+(sp ? HTMLHelper.reprocessOptions(aligned, recommendReprocess) : '')+'</td>';

					html += '<td rowspan="'+tbodySpan+'" class="mode-resolve dottedleft editable">'+(sp ? HTMLHelper.okToUseOptions(aligned, filterType) : '')+'</td>';
					html += '<td rowspan="'+tbodySpan+'" class="mode-resolve editable">'+(sp ? HTMLHelper.resolveOptions(aligned, filterType, tbodySpan==rowspan) : '')+'</td>';
					html += '<td rowspan="'+tbodySpan+'" class="mode-resolve editable">'+(sp ? HTMLHelper.futureGtOptions(aligned, filterType) : '')+'</td>';
					html += '<td rowspan="'+tbodySpan+'" class="mode-resolve comment-col dottedright editable">'+(sp ? HTMLHelper.resolveComment(aligned) : '')+'</td>';
				}
				
				var flagged = aligned.flagged;
				var isAlignment = aligned.name;
				html += '<td rowspan="'+rowspan+'" class="mode-reprocess comment-col editable">'+(isAlignment ? HTMLHelper.qcCommentInput(aligned) : '')+'</td>';				
				html += '<td rowspan="'+rowspan+'" class="mode-reprocess flag-col dottedright editable">'+(isAlignment && sp && !flagged ? HTMLHelper.qcFlagCheckbox(aligned) : '')+'</td>';
			}
			html += '<td rowspan="'+rowspan+'" class="icon-col">'+tmp.icons+'</td>';

			html += '<td rowspan="'+rowspan+'" class="dottedleft bg-messages">'+tmp.tooltip+'</td>';
			// The first message is on the main <tr>
			if (cmp && cmp.messages.length > 0)
			{
				var msg = cmp.messages[0];
				html += search.getMessageHtml(msg, aligned, assaysByName, qiacubefilter, libplatefilter);
			}
			else
			{
				html += '<td class="bg-messages"></td><td class="dottedleft" colspan="5"></td>';
			}
			html += '</tr>';
			
			// If there are more messages, add more <tr>
			if (cmp)
			{
				for (var mNo = 1; mNo < cmp.messages.length; mNo++)
				{
					var msg = cmp.messages[mNo];
					html += '<tr>'+search.getMessageHtml(msg, aligned, assaysByName, qiacubefilter, libplatefilter) + '</tr>';
				}
			}
			
			if (tb.end)
			{
				html += '</tbody>';
			}
		}
		
		if (numListed == 0)
		{
			html += '<tbody><tr><td colspan="20">No alignments matched the filter</td></tr></tbody>';
			Doc.element('mode-genotype').checked = true;
		}
		
		Doc.element('itemTable').innerHTML = html;
		Doc.element('step-1-title').innerHTML = stepTitle + ' (' + numListed + ')';
		
		Doc.show('step-1');
		
		var enableReprocess = numListed > 0 && filter && !noContextAnalysis;
		var enableResolve = enableReprocess;
		Doc.element('mode-reprocess').disabled = !enableReprocess;
		Doc.element('mode-resolve').disabled = !enableResolve;
		search.modeOnChange();

		var cs = document.getElementsByClassName('case-summary');
		for (var i = 0; i < cs.length; i++)
		{
			Events.addEventHandler(cs[i], 'click', Reggie.openCaseSummaryPopupOnEvent);
		}
		
		var lnk = document.getElementsByClassName('qiacube-link');
		for (var i = 0; i < lnk.length; i++)
		{
			Events.addEventHandler(lnk[i], 'click', search.setQiaCubeFilter);
		}
		var lnk = document.getElementsByClassName('libplate-link');
		for (var i = 0; i < lnk.length; i++)
		{
			Events.addEventHandler(lnk[i], 'click', search.setLibPlateFilter);
		}
		
		var cm = document.getElementsByClassName('colormatch');
		for (var i = 0; i < cm.length; i++)
		{
			Events.addEventHandler(cm[i], 'click', search.colorMatch);
		}
		
		var options = document.getElementsByClassName('reprocess-options');
		for (var i = 0; i < options.length; i++)
		{
			Events.addEventHandler(options[i], 'change', search.reprocessOptionOnChange);
		}
		
		var options = document.getElementsByClassName('donotuse-options');
		for (var i = 0; i < options.length; i++)
		{
			Events.addEventHandler(options[i], 'change', search.doNotUseOptionOnChange);
		}

		var checks = document.getElementsByClassName('oktouse');
		for (var i = 0; i < checks.length; i++)
		{
			Events.addEventHandler(checks[i], 'change', search.okToUseOnChange);
		}
		
		// Reset the change detection flags.
		Wizard.setNoConfirmOnFirstStep(true);
		numChangedFormElements = 0;
	}
	
	
	/**
		Primary analysis of of problems related to alignments that
		can be determined without knowledge of context and without need
		to find information about related (same patient) alignments.
	*/
	search.primaryAnalysis = function()
	{
		// Reset this mapping, since we are only interested in alignments in the 
		// current context (eg. QiaCube run or LibPlate)
		alignmentsByPat = {};
		
		for (var itemNo = 0; itemNo < alignments.length; itemNo++)
		{
			var aligned = alignments[itemNo];

			// Reset bug info
			aligned.tmpId = itemNo; // Just to give each item a unique identified that we can use in gui form elements
			aligned.bugged = 0;
			aligned.buglevel = 0;
			aligned.bugText = [];
			
			// We need to be able to lookup alignments by name and patient name
			if (!aligned.specimen)
			{
				aligned.bugText[aligned.bugText.length] = 'External';
			}
			else
			{
				var patName = aligned.specimen.patientName;
				var alignmentsForPat = alignmentsByPat[patName] || [];
				if (alignmentsForPat.length == 0) alignmentsByPat[patName] = alignmentsForPat;
				alignmentsForPat[alignmentsForPat.length] = aligned;
			}
			if (aligned.name)
			{
				assaysByName[aligned.name] = aligned;
				if (aligned.QC_GENOTYPE_STATUS != 'Checked')
				{
					aligned.bugText[aligned.bugText.length] = 'Genotyping is disabled for this alignment';
				}
			}
			else
			{
				aligned.bugText[aligned.bugText.length] = aligned.lib ? 
					'No alignment from this Library' : 'No alignment from this RNA';
				var libOrRna = aligned.lib || aligned.rna || {};
				// Copy DO_NOT_USE information to make it easier to handle in other places
				aligned.DO_NOT_USE = libOrRna.DO_NOT_USE;
				aligned.DO_NOT_USE_COMMENT = libOrRna.DO_NOT_USE_COMMENT
			}
			
			var cmp = aligned.compare;
			if (cmp)
			{
				if (cmp.hasVerifiedMismatch)
				{
					aligned.bugged = 1;
					if (aligned.QC_GENOTYPE_VERIFIED)
					{
						// This is strange -- two VERIFIED alignments with a mismatch!!
						aligned.buglevel = 2;
						aligned.bugText[aligned.bugText.length] = 'Verified × 2 mismatch!!';
					}
					else
					{
						aligned.buglevel = 1;
						aligned.bugText[aligned.bugText.length] = 'Verified mismatch';
					}
				}
			}
			
			if (aligned.QC_GENOTYPE_HET_PCT > 65 && !aligned.QC_GENOTYPE_VERIFIED)
			{
				aligned.bugged = 1;
				aligned.buglevel = 1;
				aligned.bugText[aligned.bugText.length] = 'High HET';
			}
			
			if (aligned.DO_NOT_USE)
			{
				var doNotUseFlags = {};
				doNotUseFlags[aligned.DO_NOT_USE] = 1;
				aligned.bugText[aligned.bugText.length] = search.doNotUseMessage(aligned);
				search.addDoNotUseMessageIfDifferent(aligned, aligned.lib, doNotUseFlags);
				search.addDoNotUseMessageIfDifferent(aligned, aligned.rna, doNotUseFlags);
				search.addDoNotUseMessageIfDifferent(aligned, aligned.lysate, doNotUseFlags);
				search.addDoNotUseMessageIfDifferent(aligned, aligned.specimen, doNotUseFlags);
				if (!aligned.bugged)
				{
					aligned.bugged = 1;
					aligned.buglevel = 1;
				}
			}
			
			// Copy information about current re-processing flag to the 'alignment' object to make it easier to find later
			var currentReProcess = null;
			if (aligned.specimen && aligned.specimen.AutoProcessing == 'ReProcess')
			{
				currentReProcess = 'Specimen';
			}
			else if (aligned.lysate && aligned.lysate.AutoProcessing == 'ReProcess')
			{
				currentReProcess = 'Lysate';
			}
			else if (aligned.rna && aligned.rna.AutoProcessing == 'ReProcess')
			{
				currentReProcess = 'RNA';
			}
			aligned.currentReProcess = currentReProcess;
		}
	}
	
	search.doNotUseMessage = function(item)
	{
		return '<span class="donotuse">['+Strings.encodeTags(item.DO_NOT_USE)+']</span> ' + Strings.encodeTags(item.DO_NOT_USE_COMMENT);
	}
	
	// Add 'bugtext' messages about DO_NOT_USE, but only one message for each reason
	search.addDoNotUseMessageIfDifferent = function(aligned, item, doNotUseFlags)
	{
		if (item && item.DO_NOT_USE && !doNotUseFlags[item.DO_NOT_USE])
		{
			doNotUseFlags[item.DO_NOT_USE] = 1;
			aligned.bugText[aligned.bugText.length] = search.doNotUseMessage(item);
		}
	}
	
	/**
		Analysis of problems related to a given context (eg. a QiaCube or LibPlate).
		The context is given by a all alignments matching the given filter.
		
		1. For each alignment in the context we will search for a LOW_MISMATCH warning.
		2. A LOW_MISMATCH means that we have two similar alignments that have different patients (very unlikely!)
		3. So, we take that other patient and search for other alignments in the given context that
		   have the same patient.
		4. If we find another alignment this could be a possible mixup with that position.
	*/
	search.contextAnalysis = function(filter, positionProperty)
	{
		for (var itemNo = 0; itemNo < alignments.length; itemNo++)
		{
			var aligned = alignments[itemNo];
			if (!aligned.compare) continue; // No alignment/genotype data in this position, skip ahead
			
			var filterValue = filter(aligned);
			if (filterValue == null) continue; // Not in the given context
			
			var cmp = aligned.compare;
			var possiblePositions = [];
			var possiblePats = [];
			
			for (var mNo = 0; mNo < cmp.messages.length; mNo++)
			{
				var msg = cmp.messages[mNo];
				if (msg.category != 'LOW_MISMATCH') continue; // We are only interested in 'LOW_MISMATCH' warnings
				
				var otherA = assaysByName[msg.otherAssay];
				if (!otherA) continue;
				if (otherA.bugged && otherA.buglevel > 0) continue; // The other assay is a known problem, so we ignore it here
				
				// If the other alignment is in the same context it may not be useful unless it is a verified alignment
				if (filter(otherA) && !otherA.QC_GENOTYPE_VERIFIED) continue;
				
				// This is our candidate patient
				var otherPat = otherA.specimen.patientName;
				
				// Find alignments in the same context for the candiate patient
				var alignmentsForPat = alignmentsByPat[otherPat] || [];
				for (var i = 0; i < alignmentsForPat.length; i++)
				{
					var otherB = alignmentsForPat[i];
					var posB = positionProperty(otherB);
					if (filter(otherB) && possiblePositions.indexOf(posB) == -1)
					{
						// Save the position (eg, Plate coordinate or QiaCube position)
						possiblePositions[possiblePositions.length] = posB;
					}
				}
				if (possiblePats.indexOf(otherPat) == -1) possiblePats[possiblePats.length] = otherPat;
			}
			
			if (possiblePositions.length > 0)
			{
				// If we have found other positions, we report them to the user
				aligned.bugged = 1;
				var text = 'Is this from ';
				for (var posNo = 0; posNo < possiblePositions.length; posNo++)
				{
					var pos = possiblePositions[posNo];
					if (posNo > 0) text += posNo == possiblePositions.length-1 ? ' or ' : ', ';
					text += HTMLHelper.colorMatch(filterValue + '@' + pos, '@'+pos, 'Click to highlight this position');
				}
				text += '/';
				for (var patNo = 0; patNo < possiblePats.length; patNo++)
				{
					var pat = possiblePats[patNo];
					if (patNo > 0) text += patNo == possiblePats.length-1 ? ' or ' : ', ';
					text += HTMLHelper.colorMatch(pat, null, 'Click to highlight this patient');
				}
				text += '?';
				aligned.bugText[aligned.bugText.length] = text;
			}
			else if (possiblePats.length > 0)
			{
				// We found a low mismatch but not any other sample with the same patient
				// The mixup is probably not in the selected context
				var text = 'Could not find sample related to ';
				for (var patNo = 0; patNo < possiblePats.length; patNo++)
				{
					var pat = possiblePats[patNo];
					if (patNo > 0) text += patNo == possiblePats.length-1 ? ' or ' : ', ';
					text += HTMLHelper.colorMatch(pat, null, 'Click to highlight this patient');
				}
				aligned.bugText[aligned.bugText.length] = text;
			}
		}
		
	}
	
	/**
		EVent handler for highlighting related items with the same
		colormatch tag. For all items tagged with 'colormatch' we highlight
		all elements with the same 'match' value as the clicked element.
		Two different colors are supported by holding down CTRL, ALT or SHIFT
		while clicking.
	*/
	search.colorMatch = function(event)
	{
		var toMatch = Data.get(event.currentTarget, 'match');
		var specialKey = event.altKey || event.ctrlKey || event.shiftKey;
		var matchedClass = specialKey ? 'colormatched-2' : 'colormatched';
		var cm = document.getElementsByClassName('colormatch');
		var add = !event.currentTarget.classList.contains(matchedClass);
		for (var i = 0; i < cm.length; i++)
		{
			var e = cm[i];
			Doc.addOrRemoveClass(e, matchedClass, Data.get(e, 'match') == toMatch && add);
		}
	}
	
	/**
		Display a warning icon if the selected re-process option is marked 'donotuse'.
	*/
	search.reprocessOptionOnChange = function(event)
	{
		var opt = event.currentTarget[event.currentTarget.selectedIndex];
		var tmpId = event.currentTarget.id.replace('reprocess.', '');
		Doc.addOrRemoveClass('section.'+tmpId, 'reprocess-warning', opt.classList.contains('donotuse'));
	}
	
	// Enable/disable the DoNotUse list depending on if the OkToUse is checked or not
	search.okToUseOnChange = function(event)
	{
		var tmpId = event.currentTarget.id.replace('oktouse.', '');
		var doNotUseList = Doc.element('donotuse.'+tmpId);
		var futureGtCheck = Doc.element('futuregt.'+tmpId);
		var commentField = Doc.element('resolvecomment.'+tmpId);
		if (event.currentTarget.checked)
		{
			doNotUseList.selectedIndex = 0;
			Events.sendChangeEvent(doNotUseList);
			Doc.removeClass(doNotUseList, 'required');
			Doc.removeClass(commentField, 'required');
			doNotUseList.disabled = true;
			futureGtCheck.disabled = true;
		}
		else
		{
			Doc.addClass(doNotUseList, 'required');
			doNotUseList.disabled = false;
		}
	}

	
	// Mark the corresponding comment field as required if the DoNotUse flag is going to be set
	search.doNotUseOptionOnChange = function(event)
	{
		var tmpId = event.currentTarget.id.replace('donotuse.', '');
		var value = event.currentTarget.value;		
		var futureGtCheck = Doc.element('futuregt.'+tmpId);
		if (value == '')
		{
			futureGtCheck.checked = false;
			Events.sendChangeEvent(futureGtCheck);
		}
		futureGtCheck.disabled = value == '';
		var commentField = Doc.element('resolvecomment.'+tmpId);
		Doc.addOrRemoveClass(commentField, 'required', value != '');
	}
	
	search.getAlignmentHtml = function(alignment, refAlignment)
	{
		var tmp = {};
		
		var itemName = alignment.name;
		var suffix = '';
		var markedName = refAlignment ? HTMLHelper.tagCommonPartOfName(refAlignment.name, itemName) : null;
		
		if (!itemName)
		{
			var subtype = null;
			if (alignment.lib)
			{
				itemName = alignment.lib.name;
				subtype = 'Library';
			}
			else if (alignment.rna)
			{
				itemName = alignment.rna.name;
				subtype = 'RNA';
			}
			markedName = itemName;
			if (subtype)
			{
				suffix = ' <span class="itemsubtype">('+subtype+')</span>';
			}
		}
		
		tmp.name = HTMLHelper.colorMatch(itemName, markedName, 'Click to highlight this item') + suffix;
		
		var icons = '';
		var tooltip = '';
		
		if (alignment.specimen)
		{
			tmp.name += '<img src="../images/case_summary.png" class="link case-summary" data-name="'+Strings.encodeTags(itemName)+'" title="Show case summary">';
		}
		if (alignment.QC_GENOTYPE_COMMENT)
		{
			icons += '<img src="../images/comment.png" title="'+Strings.encodeTags(alignment.QC_GENOTYPE_COMMENT)+'">';
		}
		if (alignment.DO_NOT_USE)
		{
			icons += '<img src="../images/donotuse.png" title="DoNotUse-'+Strings.encodeTags(alignment.DO_NOT_USE+': '+alignment.DO_NOT_USE_COMMENT)+'">';
		}
		if (alignment.currentReProcess)
		{
			icons += '<img src="../images/reprocess.png" title="Re-processing is scheduled from '+alignment.currentReProcess+'">';
		}
		
		if (alignment.flagged)
		{
			// We need this class to search for flagged alignments on the "right" side of the table (see goResolve() method)
			var cls = refAlignment ? 'paired-alignment' : 'primary-alignment';
			icons += '<img src="../images/flag.png" title="This alignment is already flagged" class="'+cls+'" data-id="'+alignment.id+'">';
		}
		if (alignment.QC_GENOTYPE_VERIFIED) 
		{
			var img = alignment.QC_GENOTYPE_VERIFIED == 'Lysate' ? 'gt-verified-lysate.png' : 'gt-verified.png';
			tooltip = Strings.encodeTags('Verified by '+alignment.QC_GENOTYPE_VERIFIED);
			icons += '<img src="../images/'+img+'" title="'+tooltip+'">';
		}
		if (alignment.bugged)
		{
			var img = bugIcons[alignment.buglevel];
			// .replace will strip simple HTML tags
			var imgTitle = alignment.buglevel > 0 && alignment.bugText.length > 0 ? alignment.bugText[0].replace(/<\/?[^>]+(>|$)/g, '') : '';
			icons += '<img src="../images/'+img+'" title="'+imgTitle+'">';
		}
		if (alignment.bugText && alignment.bugText.length > 0)
		{
			tooltip = alignment.bugText.join('<br>');
		}
		
		tmp.icons = icons;
		tmp.tooltip = tooltip;
		return tmp;
	}
	
	search.getMessageHtml = function(message, aligned, assaysByName, qiacubefilter, libplatefilter)
	{
		var html = '';
		var msgClass= "bg-messages";
		if (message.otherAssay)
		{
			html += '<td class="dottedleft text-col">';
			var otherAssay = assaysByName[message.otherAssay];

			var tmp = search.getAlignmentHtml(otherAssay, aligned);
			html += tmp.name+tmp.icons;

			if (otherAssay)
			{
				if (otherAssay.buglevel == 1 && (qiacubefilter || libplatefilter)) msgClass += ' ignored-mismatch';
				html += '<td>'+Reggie.formatNumber(otherAssay.QC_GENOTYPE_HET_PCT, '%', -1)+'</td>';
				var rna = otherAssay.rna || {};
				var dna = otherAssay.dna || {};
				var lib = otherAssay.lib;
				var chip = otherAssay.chip;
				var specimen = otherAssay.specimen || {};
				
				html += '<td class="mode-genotype">'+HTMLHelper.qiaCubeLink(rna.QIACUBE_DATE, rna.QIACUBE_RUN_NO, rna.QIACUBE_POSITION, qiacubefilter, false)+'</td>';
				html += '<td class="mode-genotype">';
				if (lib)
				{
					var libWell = lib.bioWell || {};
					var libPlate = libWell.bioPlate || {};
					html +=  HTMLHelper.libPlateLink(libPlate.name, libWell.location, libplatefilter, false);
				}
				else if (chip)
				{
					html += HTMLHelper.beadChipLink(chip.name, dna.BeadChipPosition);
				}
				html += '</td>';
				html += '<td>'+HTMLHelper.colorMatch(specimen.patientName, null, 'Click to highlight this patient')+'</td>';
			}
		}
		else 
		{
			html += '<td class="dottedleft" colspan="5"></td>';
		}
		html = '<td class="' + msgClass+ '" title="'+(message.messageTooltip || '')+'">'+Strings.encodeTags(message.message) + '</td>' + html;
		return html;
	}
	
	search.modeOnChange = function()
	{
		var frm = document.forms['reggie'];
		var mode = Forms.getCheckedRadio(frm.mode).value;
		Data.set('itemTable', 'mode', mode);
		Doc.showHide('goreprocess', mode == 'reprocess');
		Doc.showHide('goresolve', mode == 'resolve');
		Wizard.hideGoNextConfirmation();
	}
	
	search.goReProcess = function()
	{
		Wizard.hideGoNextConfirmation();
		Doc.hide('wizard-status');
		var frm = document.forms['reggie'];
		var items = [];
		for (var itemNo = 0; itemNo < alignments.length; itemNo++)
		{
			var aligned = alignments[itemNo];
			
			// Get the form elements for this item
			var listE = Doc.element('reprocess.'+aligned.tmpId);
			var flagE = Doc.element('flag.'+aligned.tmpId);
			var commentE = Doc.element('comment.'+aligned.tmpId);
			
			var item = { 'id': aligned.id };
			var changed = false;
			
			// Check if a re-processing option was selected
			if (listE && listE.value)
			{
				changed = true;
				if (listE.value == 'specimen')
				{
					item.reprocess = {'type': 'SAMPLE', 'id': aligned.specimen.id };
				}
				else if (listE.value == 'lysate')
				{
					item.reprocess = {'type': 'EXTRACT', 'id': aligned.lysate.id };
				}
				else if (listE.value == 'rna')
				{
					item.reprocess = {'type': 'EXTRACT', 'id': aligned.rna.id };
				}
			}
			if (flagE && flagE.checked)
			{
				changed = true;
				item.flag = true;
			}
			if (commentE && Strings.trim(commentE.value) != Strings.trim(aligned.QC_GENOTYPE_COMMENT))
			{
				changed = true;
				item.comment = commentE.value;
			}
			
			if (changed) items[items.length] = item;
		}
		
		if (items.length == 0)
		{
			Wizard.showGoNextConfirmation(false, 'No changes have been made!');
			return;
		}
		
		Doc.addClass('step-1', 'disabled');
		Doc.hide('navigation');
		
		
		// Include filter settings in the registration URL so that a new list can be generated
		var highHet = Doc.element('highHetFilter');
		var libPlate = Doc.element('libPlateFilter');
		var qiaCube = Doc.element('qiaCubeFilter');
		
		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=ReprocessItems';
		if (libPlate.selectedIndex > 0)
		{
			url += '&libPlateFilter='+encodeURIComponent(libPlate.value);
		}
		else if (qiaCube.selectedIndex > 0)
		{
			url += '&qiaCubeFilter='+encodeURIComponent(qiaCube.value);
		}
		else if (highHet.checked)
		{
			url += '&highHetFilter=1';
		}
		
		var submitInfo = {};
		submitInfo.items = items;
		Wizard.showLoadingAnimation('Saving changes...');
		Wizard.asyncJsonRequest(url, search.reprocessResults, 'POST', JSON.stringify(submitInfo));
	}

	search.reprocessResults = function(response)
	{
		search.onAlignmentsLoaded(response);
		Doc.show('navigation');
		Doc.removeClass('step-1', 'disabled');
		Wizard.showFinalMessage(response.messages);
	}
	
	
	search.goResolve = function()
	{
		var verifyCheck = Doc.element('verifyGoNext');
		var acceptPartial = verifyCheck !=null && verifyCheck.checked;
		
		Wizard.hideGoNextConfirmation();
		Doc.hide('wizard-status');
		var frm = document.forms['reggie'];
		// We generate a list of items that should either be marked as DoNotUse or OkToUse
		var items = [];
		// This list will are flagged alignements that have a relation to the items in the above list
		// The are potential candidates to be unflagged due to not having any more issues with other alignments
		var flaggedPairs = [];
		var numInvalid = 0;
		var numCanAllowIfPartial = 0;
		
		for (var itemNo = 0; itemNo < alignments.length; itemNo++)
		{
			var aligned = alignments[itemNo];
			
			// Assume that selection is valid
			Doc.removeClass('section.'+aligned.tmpId, 'invalid');
			
			// Get the form elements for this item
			var okToUse = Doc.element('oktouse.'+aligned.tmpId);
			var doNotUse = Doc.element('donotuse.'+aligned.tmpId);
			var futureGt = Doc.element('futuregt.'+aligned.tmpId);
			var comment = Doc.element('resolvecomment.'+aligned.tmpId);
			
			var item = null;
			if (doNotUse)
			{
				if (okToUse && okToUse.checked)
				{
					if (okToUse.value == 'ok-rna')
					{
						item = {'type': 'EXTRACT', 'id': aligned.rna.id, 'okToUse': true, 'sort': 6 };
					}
					else if (okToUse.value == 'ok-library')
					{
						item = {'type': 'EXTRACT', 'id': aligned.lib.id, 'okToUse': true, 'sort': 7 };
					}
				}
				else if (doNotUse.value == 'specimen')
				{
					item = {'type': 'SAMPLE', 'id': aligned.specimen.id, 'doNotUse': 'Specimen', 'sort': 1 };
				}
				else if (doNotUse.value == 'lysate')
				{
					item = {'type': 'EXTRACT', 'id': aligned.lysate.id, 'doNotUse': 'Lysate', 'sort': 2 };
				}
				else if (doNotUse.value == 'rna')
				{
					item = {'type': 'EXTRACT', 'id': aligned.rna.id, 'doNotUse': 'RNA', 'sort': 3 };
				}
				else if (doNotUse.value == 'prenormalized')
				{
					item = {'type': 'EXTRACT', 'id': aligned.preNormalizedRna.id, 'doNotUse': 'PreNormalizedRNA', 'sort': 4 };
				}
				else if (doNotUse.value == 'library')
				{
					item = {'type': 'EXTRACT', 'id': aligned.lib.id, 'doNotUse': 'Library', 'sort': 5 };
				}
				else if (aligned.flagged && doNotUse.classList.contains('required'))
				{
					// A flagged item must have a specific option selected (when filtered on QiaCube or LibPlate)
					numCanAllowIfPartial++;
					if (!acceptPartial)
					{
						Doc.addClass('section.'+aligned.tmpId, 'invalid');
						numInvalid++;
					}
				}
				if (doNotUse.value && futureGt)
				{
					item.futureGt = futureGt.checked;
				}
			}
			if (item && comment)
			{
				item.comment = Strings.trim(comment.value);
				if (item.doNotUse && item.comment == '')
				{
					// A comment must be specified for all items marked with DoNotUse
					Doc.addClass('section.'+aligned.tmpId, 'invalid');
					numInvalid++;
				}
			}
			if (item != null)
			{
				items[items.length] = item;
				
				// Find flagged alignments that are related to the selected item
				// We take help from the <tbody> section and look for tags with 'paired-alignment' class
				var sectR = Doc.element('section.'+aligned.tmpId);
				var flagged = sectR.getElementsByClassName('paired-alignment');
				for (var fNo = 0; fNo < flagged.length; fNo++)
				{
					flaggedPairs[flaggedPairs.length] = Data.int(flagged[fNo], 'id');
				}
			}
		}

		if (numInvalid > 0)
		{
			if (numCanAllowIfPartial > 0)
			{
				Wizard.showGoNextConfirmation(true, 'Mark the checkbox to allow partial registration.', acceptPartial);
			}
			Wizard.notifyError();
			return;
		}
		
		if (items.length == 0)
		{
			Wizard.showGoNextConfirmation(false, 'No changes have been made!');
			return;
		}

		Doc.addClass('step-1', 'disabled');
		Doc.hide('navigation');
		
		// Include filter settings in the registration URL so that a new list can be generated
		var highHet = Doc.element('highHetFilter');
		var libPlate = Doc.element('libPlateFilter');
		var qiaCube = Doc.element('qiaCubeFilter');
		
		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=ResolveProblems';
		if (libPlate.selectedIndex > 0)
		{
			url += '&libPlateFilter='+encodeURIComponent(libPlate.value);
		}
		else if (qiaCube.selectedIndex > 0)
		{
			url += '&qiaCubeFilter='+encodeURIComponent(qiaCube.value);
		}
		else if (highHet.checked)
		{
			url += '&highHetFilter=1';
		}
		
		// Sort items in the "correct" order. All DoNotUse items before OkToUse
		// and Specimen -> Lysate -> RNA -> Library
		// This will prevent prevent problems with non-deterministic result if
		// selecting, for example, "Specimen" and "RNA" from the same parent-child chain
		// With sorting the highest level (Specimen) wins
		items.sort(Sort.sortDoNotUseItems);
		
		var submitInfo = {};
		submitInfo.items = items;
		submitInfo.paired = flaggedPairs;	
		
		Wizard.showLoadingAnimation('Saving changes...');
		Wizard.asyncJsonRequest(url, search.resolveResults, 'POST', JSON.stringify(submitInfo));
	}
	
	search.resolveResults = function(response)
	{
		search.onAlignmentsLoaded(response);
		Doc.show('navigation');
		Doc.removeClass('step-1', 'disabled');
		Wizard.showFinalMessage(response.messages);
	}

	return search;
}();

Doc.onLoad(Search.initPage);

/**
	Represents a QiaCube in the QiaCube filter list.
	The date and run number is the important information.
*/
function QiaCube(date, runNo)
{
	this.date = date;
	this.runNo = runNo;
	this.name = date + '#' + runNo;
	this.title = Reggie.reformatDate(date) + '#' + runNo;
}

/**
	Represents a LibPlate in the LibPlate filter list.
	The name is the only important information.
*/
function LibPlate(name)
{
	this.name = name;
}

// Placeholder for various sort functions
var Sort = function()
{
	var sort = {};
	
	// Sort items by name
	sort.sortByName = function(a, b)
	{
		return a.name == b.name ? 0 : (a.name < b.name ? -1 : 1);
	}

	// Sort alignments by patient name
	// If they are equals sort by alignment name
	sort.sortAlignmentsByPatientName = function(a, b)
	{
		var patA = a.specimen ? a.specimen.patientName : 'zz'+(a.name || a.rna.name);
		var patB = b.specimen ? b.specimen.patientName : 'zz'+(b.name || b.rna.name);
		return patA == patB ? sort.sortByName(a, b) : (patA < patB ? -1 : 1);
	}
	
	// Sort alignments by LibPlate position (columns before rows)
	sort.sortAlignmentsByLibPlatePosition = function(a, b)
	{
		var plateA = a.lib ? a.lib.bioWell.bioPlate.name : 'zz'+(a.name || a.rna.name);
		var plateB = b.lib ? b.lib.bioWell.bioPlate.name : 'zz'+(b.name || b.rna.name);

		if (plateA != plateB) return plateA < plateB ? -1 : 1;
		
		var posA = a.lib.bioWell;
		var posB = b.lib.bioWell;
		if (posA.column != posB.column) return posA.column < posB.column ? -1 : 1;
		
		return posA.row < posB.row ? -1 : 1;
	}
	
	// Sort alignments by QiaCube position
	sort.sortAlignmentsByQiaCubePosition = function(a, b)
	{
		var qiaCubeA = a.rna.QIACUBE_DATE + '#' + a.rna.QIACUBE_RUN_NO;
		var qiaCubeB = b.rna.QIACUBE_DATE + '#' + b.rna.QIACUBE_RUN_NO;

		if (qiaCubeA != qiaCubeB) return qiaCubeA < qiaCubeB ? -1 : 1;

		var posA = a.rna.QIACUBE_POSITION;
		var posB = b.rna.QIACUBE_POSITION;
		if (posA != posB) return posA < posB ? -1 : 1;
		
		// Sort flagged before not flagged
		if (a.flagged != b.flagged) return a.flagged ? -1 : 1;
		
		// Critical, sort pre-normalized RNA before regular RNA to
		// enable the 'PreNormalizedRNA' option in the 'DoNotUse' selection list
		if (a.preNormalizedRna && !b.preNormalizedRna) return -1;
		if (b.preNormalizedRna && !a.preNormalizedRna) return 1;
		
		return sort.sortByName(a, b);
	}
	
	// Sort items that are sent for "DoNotUse" or "OkToUse" by the 'sort' propert.
	sort.sortDoNotUseItems = function(a, b)
	{
		return a.sort == b.sort ? a.id - b.id : a.sort - b.sort;
	}
	
	return sort;
}();

//Placeholder for various filter functions
var Filter = function()
{
	var filter = {};
	
	// Creates a filter that accepts alignments related to
	// the given QiaCube run. The filter value should
	// be formatted as <date>#<runNo>
	filter.qiaCubeFilter = function(qiaCubeRun)
	{
		return function(assay)
		{
			var rna = assay.rna;
			return rna && qiaCubeRun == rna.QIACUBE_DATE + '#' + rna.QIACUBE_RUN_NO ? qiaCubeRun : null;
		}
	}
	
	// Creates a filter that accepts alignments related
	// to the given LibPlate. The filter value is the name
	// of the plate,
	filter.libPlateFilter = function(libPlateName)
	{
		return function(assay)
		{
			return assay.lib && libPlateName == assay.lib.bioWell.bioPlate.name ? libPlateName : null;
		}
	}
	
	// Creates a filter that accepts alignments with a HET
	// percentage above the specified limit
	filter.hetFilter = function(hetLimit)
	{
		return function(assay)
		{
			return assay.QC_GENOTYPE_HET_PCT > hetLimit;
		}
	}
	
	return filter;
}();

// Placeholder for various functions that help us get a property from an alignment 
var Property = function()
{
	var property = {};
	
	// Get the QiaCube position from the alignment
	property.QiaCubePosition = function(alignment)
	{
		return alignment.rna.QIACUBE_POSITION;
	}
	
	// Get the LibPlate position from the alignment
	property.LibPlatePosition = function(alignment)
	{
		return alignment.lib.bioWell.location;
	}
	
	return property;
}();

/**
	Helper functions for generating various HTML outputs.
*/
var HTMLHelper = function()
{
	var helper = {};
	
	var colorMatchTargets = {};
	var colorMatchIndex = 0;
	
	/**
		Create the link for a QiaCube run. It will typically show the date#run@pos
		and will also output colormatch functionality for either the entire QiaCube run 
		or the position (in filtered view)
	*/
	helper.qiaCubeLink = function(qiaCubeDate, qiaCubeRun, qiaCubePos, filterValue, shortForm)
	{
		if (!qiaCubeDate) return '';
		var runId = qiaCubeDate+'#'+qiaCubeRun;
		var runPosId = runId+'@'+qiaCubePos;
		var html = '';
		if (!shortForm)
		{
			html += '<span class="link qiacube-link" data-qiacuberun="'+runId+'" title="Filter on this QiaCube run">';
			html += Reggie.reformatDate(qiaCubeDate) + '#' + qiaCubeRun;
			html += '</span>';
		}
		html += '@'+qiaCubePos;
		var isFiltered = filterValue == runId;
		return HTMLHelper.colorMatch(isFiltered ? runPosId : runId, html, isFiltered ? 'Click to highlight this position' : 'Click to highlight this QiaCube');
	}
	
	/**
		Create the link for a LibPlate position. It will typically show the name and position
		and will also output colormatch functionality for either the entire LibPlate 
		or the position (in filtered view).
	*/
	helper.libPlateLink = function(libPlateName, libPlatePos, filterValue, shortForm)
	{
		if (!libPlateName) return '';
		var html = '';
		if (!shortForm)
		{
			html = '<span class="link libplate-link" data-libplate="'+libPlateName+'" title="Filter on this plate">';
			html += Strings.encodeTags(libPlateName);
			html += '</span>&nbsp;';
		}
		html += libPlatePos;
		var isFiltered = filterValue == libPlateName;
		return HTMLHelper.colorMatch(isFiltered ? libPlateName+'@'+libPlatePos : libPlateName, html, isFiltered ? 'Click to highlight this position' : 'Click to highlight this LibPlate');
	}
	
	helper.beadChipLink = function(beadChipName, beadChipPosition)
	{
		if (!beadChipName) return '';
		var html = Strings.encodeTags(beadChipName)+'&nbsp;'+Strings.encodeTags(beadChipPosition);
		return HTMLHelper.colorMatch(beadChipName, html, 'Click to highlight this BeadChip');
	}
	
	/**
		Return 'name2' with markup indicating which parts of the name
		are common with 'name1'.
	*/
	helper.tagCommonPartOfName = function(name1, name2)
	{
		var n1 = name1.split('.');
		var n2 = name2.split('.');
		var i = 0;
		
		var common = '';
		var sep = '.';
		while (i < n1.length && i < n2.length && n1[i] == n2[i])
		{
			common += n1[i] + sep;
			i++;
		}
		
		var diff = '';
		sep = '';
		while (i < n2.length)
		{
			diff += sep + n2[i];
			i++;
			sep = '.';
		}
		
		var html = '';
		if (common.length > 0) html += '<span class="commonpart">'+Strings.encodeTags(common)+'</span>';
		if (diff.length > 0) html += '<span class="diffpart">'+Strings.encodeTags(diff)+'</span>';
		return html;
	}

	
	// Creates a "ColorMatch" tag for the given text
	helper.colorMatch = function(plainText, formattedText, tooltip)
	{
		var target = colorMatchTargets[plainText];
		if (!target)
		{
			colorMatchIndex++;
			target = 'CM'+colorMatchIndex;
			colorMatchTargets[plainText] = target;
		}
		if (!formattedText) formattedText = Strings.encodeTags(plainText);

		var html = '<span class="colormatch" data-match="'+target+'"';
		if (tooltip) html += ' title="'+tooltip+'"';
		html += '>'+formattedText+'</span>';
		return html;
	}
	
	
	helper.reprocessOptions = function(aligned, recommendReprocess)
	{
		var id = 'reprocess.'+aligned.tmpId;
		var title = '';
		
		var html = '';
		html += '<select id="'+id+'" class="reprocess-options" name="'+id+'" '+(recommendReprocess? 'title="Re-processing is recommended"':'')+'>';
		if (!aligned.currentReProcess) html += '<option value="" selected>&nbsp;';
		html += helper.reprocessOption('Specimen', 'specimen', aligned.currentReProcess, aligned.specimen.DO_NOT_USE);
		html += helper.reprocessOption('Lysate', 'lysate', aligned.currentReProcess, aligned.lysate.DO_NOT_USE);
		html += helper.reprocessOption('RNA', 'rna', aligned.currentReProcess, aligned.rna.DO_NOT_USE);
		html += '</select>';
		
		if (aligned.currentReProcess != null)
		{
			html += '<img src="../images/info.png" title="Re-processing is already scheduled from '+aligned.currentReProcess+'">';
		}
		
		return html;
	}
	
	helper.reprocessOption = function(text, value, currentReProcess, doNotUse)
	{
		var opt = '<option ';
		if (currentReProcess==text)
		{
			opt += 'value="" selected';
		}
		else
		{
			opt += 'value="'+value+'"';
		}
		if (doNotUse) opt += ' class="donotuse" title="'+text+' is marked as DoNotUse"';	
		opt += '>'+text;
		if (doNotUse) opt += ' (!)';
		return opt;
	}
	
	helper.qcCommentInput = function(aligned)
	{
		var id = 'comment.'+aligned.tmpId;
		html = '<input type="text" id="'+id+'" name="'+id+'" value="'+Strings.encodeTags(aligned.QC_GENOTYPE_COMMENT)+'">';
		return html;
	}
	
	helper.qcFlagCheckbox = function(aligned)
	{
		var id = 'flag.'+aligned.tmpId;
		html = '<input type="checkbox" id="'+id+'" name="'+id+'">';
		return html;
	}

	helper.okToUseOptions = function(aligned, filterType)
	{
		var html = '';
		if (aligned.flagged && !aligned.DO_NOT_USE) 
		{
			var idOkToUse = 'oktouse.'+aligned.tmpId;
			var okValue = filterType == 'QIACUBE' ? 'ok-rna' : 'ok-library';
			html += '<input type="checkbox" value="'+okValue+'" id="'+idOkToUse+'" name="'+idOkToUse+'" class="oktouse" title="Check this to indicate that the flagged item is ok to use.">';
		}
		return html;
	}
	
	/**
		We generate a checkbox (if the item is flagged) and a 
		selection list with "DoNotUse" options.
	*/
	helper.resolveOptions = function(aligned, filterType, singleAlignment)
	{
		var html = '';
		var idDoNotUse = 'donotuse.'+aligned.tmpId;
		var cls = aligned.flagged && filterType != 'HET' ? 'required' : '';
	
		html += '<select id="'+idDoNotUse+'" name="'+idDoNotUse+'" class="donotuse-options '+cls+'">';
		html += '<option value="">&nbsp;';
		if (filterType != 'QIACUBE' || singleAlignment)
		{
			html += '<option value="library">Library';
		}
		if (aligned.preNormalizedRna)
		{
			html += '<option value="prenormalized">PreNormalizedRNA';
		}
		html += '<option value="rna">RNA';
		html += '<option value="lysate">Lysate';
		html += '<option value="specimen">Specimen';
		html += '</select>';

		return html;
	}
	
	helper.futureGtOptions = function(aligned, filterType)
	{
		var html = '';
		var idFutureGt = 'futuregt.'+aligned.tmpId;
		html += '<input type="checkbox" id="'+idFutureGt+'" name="'+idFutureGt+'" disabled title="Check this to use the alignment in future genotype checks.">';
		return html;
	}


	helper.resolveComment = function(aligned)
	{
		var html = '';
		var id = 'resolvecomment.'+aligned.tmpId;
		html += '<input type="text" id="'+id+'" name="'+id+'" value="'+Strings.encodeTags(aligned.QC_GENOTYPE_COMMENT)+'">';
		return html;
	}

	
	return helper;
}();
