var Index = function()
{
	var index = {};
	var debug = 0;
	var myPermissions = {};
	var counterInfo = null;

	index.initPage = function()
	{
		Events.doOnEnter('caseName', index.caseSummary);
		Events.addEventHandler('caseName', 'keypress', index.testSnake);
		Buttons.addClickHandler('btnCaseSummary', index.caseSummary);
		Buttons.addClickHandler('counter-refresh', index.forceLoadCounts);
		Doc.removeClass('counter-loading', 'link');
		
		var groupMenus = document.getElementsByClassName('group-menu');
		for (var groupNo = 0; groupNo < groupMenus.length; groupNo++)
		{
			var grp = groupMenus[groupNo];
			if (!Doc.isDisabled(grp))
			{
				Buttons.addClickHandler(grp, index.showWizardGroup);
			}
		}
		
		index.initPermissions();
		index.loadCounts();
		index.updateCounterAge();
		
		var frm = document.forms['reggie'];
		frm.caseName.disabled = !App.getActiveProjectId();
	}
	
	index.showWizardGroup = function(event)
	{
		var group = Data.get(event.currentTarget, 'wizard-group');
		Data.set('wizard', 'wizard-group', group);
		var url = 'Session.servlet?ID='+App.getSessionId();
		url += '&cmd=SetLastWizardGroup';
		url += '&group='+encodeURIComponent(group);
		Wizard.asyncJsonRequest(url, null, 'POST');
	}
	
	index.initPermissions = function()
	{
		// Get permission information
		var url = 'Install.servlet?ID='+App.getSessionId();
		url += '&cmd=GetPermissions';
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, index.onPermissionsLoaded, index.onPermissionsLoaded);
		request.send(null);
	}
	
	// Callback when permission information is ready
	index.onPermissionsLoaded = function(request)
	{
		if (debug) App.debug(Strings.encodeTags(request.responseText));
		var response;
		var error = false;
		try
		{
			response = JSON.parse(request.responseText);
			if (response.status != 'ok')
			{
				error = response.message || response.stacktrace || 'Unexpected error';
			}
		}
		catch (ex)
		{
			error = ex;
		}
		
		if (error)
		{
			alert(error);
			return;
		}
		
		myPermissions = response.permissions;
		var experimentalFeatures = response.experimentalFeatures;
		var isAdmin = myPermissions['Administrator'] == 1;
		var noProjectIsActive = !App.getActiveProjectId();
		
		// Enable links based on the permission
		var wizards = document.getElementsByClassName('require-permission');
		// NOTE! Loop backwards since changing the classname 
		// of an element so that 'require-permission' no longer is included
		// removes the element from the 'wizards' list
		for (var wizardNo = wizards.length-1; wizardNo >= 0; wizardNo--)
		{
			var wizard = wizards[wizardNo];
			if (wizard.className.indexOf('experimental') >= 0)
			{
				if (experimentalFeatures[wizard.id] == 1) Doc.removeClass(wizard, 'not-implemented');
			}
			
			if (wizard.className.indexOf('not-implemented') == -1)
			{
				var role = wizard.getAttribute('data-role');
				if (noProjectIsActive)
				{
					wizard.title = 'You must select a project before using this wizard';
				}
				else if (!role || myPermissions[role] == 1 || isAdmin)
				{
					wizard.className = 'link';
					wizard.addEventListener('click', index.wizardOnClick);
				}
				else
				{
					wizard.title = role + ' permission is required to use this wizard';
				}
			}
		}
		
		if (response.fastqImportPipelineDisabled) Doc.show('fastq-import-pipeline-disabled', 'inline-block');
		if (response.hisatPipelineDisabled) Doc.show('hisat-pipeline-disabled', 'inline-block');
		if (response.legacyPipelineDisabled) Doc.show('legacy-pipeline-disabled', 'inline-block');
		if (response.stringtiePipelineDisabled) Doc.show('stringtie-pipeline-disabled', 'inline-block');
		if (response.mBafPipelineDisabled) Doc.show('mbaf-pipeline-disabled', 'inline-block');
		if (response.variantCallingPipelineDisabled) Doc.show('variant-calling-pipeline-disabled', 'inline-block');
		if (response.targetedGenotypePipelineDisabled) Doc.show('targeted-genotype-pipeline-disabled', 'inline-block');
		if (response.sspPipelineDisabled) Doc.show('ssp-pipeline-disabled', 'inline-block');
		if (response.geneReportDisabled) Doc.show('gene-report-disabled', 'inline-block');
		if (response.pilotReportDisabled) Doc.show('pilot-report-disabled', 'inline-block');
		if (response.scanbReportDisabled) Doc.show('scanb-report-disabled', 'inline-block');
		if (response.variantImportDisabled) Doc.hide('variant-import-wizard');
	}

	index.wizardOnClick = function(event)
	{
		var role = Data.get(event.currentTarget, 'role');
		var isAdmin = myPermissions['Administrator'] == 1;
		if (!role || myPermissions[role] == 1 || isAdmin)
		{
			var link = Data.get(event.currentTarget, 'link');
			location.href = link;
		}
		else
		{
			Forms.showNotification(event.currentTarget, 'You do not have permission to use this wizard!', null, null, 'left');
		}
	}

	index.forceLoadCounts = function()
	{
		Doc.hide('counter-refresh');
		Doc.show('counter-loading', 'inline');
		counterInfo = null;
		index.loadCounts(true);
	}
	
	index.loadCounts = function(forceUpdate)
	{
		var url = 'Session.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCountersAndActivityLog'
		if (forceUpdate) url += '&forceUpdate=1';
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, index.onCounted, index.onCounted);
		request.send(null);
	}
	
	index.onCounted = function(request)
	{
		if (debug) 
		{
			App.debug(Strings.encodeTags(request.responseText));
		}

		var response;
		var error = false;
		try
		{
			response = JSON.parse(request.responseText);
			if (response.status != 'ok')
			{
				error = response.message || response.stacktrace || 'Unexpected error';
			}
		}
		catch (ex)
		{
			error = ex;
		}
		if (error) 
		{
			App.debug(error);
			return;
		}
		
		counterInfo = response;
		counterInfo.infoTime = new Date().getTime();
		var counts = response.counts || {};
		
		var counters = document.getElementsByClassName('counter');
		for (var counterNo = 0; counterNo < counters.length; counterNo++)
		{
			var counter = counters[counterNo];
			var countId = Data.get(counter, 'counter');

			var count = counts[countId];
			Doc.addClass(counter, 'done');
			if (count == null)
			{
				counter.innerHTML = '-';
			}
			else
			{
				counter.innerHTML = count < 1000 ? count : Math.floor(count / 1000) + ',' + index.zeroPad(count % 1000);
			}
		}
		
		if (counts['empty-storage-boxes'] < 2 || counts['empty-paused-storage-boxes'] < 2)
		{
			Doc.show('more-boxes-are-needed', 'inline-block');
		}
		
		if (!counterInfo.isRunning)
		{
			Doc.element('counter-age').innerHTML = '<img src="images/warning.png"> The counter service is not running';
			Doc.element('counter-age').title = 'Go to Administrate › Services to start the service.';
			Doc.hide('counter-loading');
			Doc.hide('counter-refresh');
		}
		else
		{
			index.displayCounterAge();
		}
		
		var activityLog = response.activityLog;
		if (activityLog)
		{
			var html = '';
			var currentDay = '';
			for (var activityNo = 0; activityNo < activityLog.length; activityNo++)
			{
				var activity = activityLog[activityNo];
				if (activity.day != currentDay)
				{
					html += '<div class="activity-day">'+Strings.encodeTags(activity.day)+'</div>';
					currentDay = activity.day;
				}
				html += '<div class="activity '+(activity.subclass||'')+'">';
				html += '<span class="activity-time">'+Strings.encodeTags(activity.time)+'</span>';
				html += '<span class="activity-message">' + Strings.encodeTags(activity.message) + '</span>';
				if (activity.quoteOfTheDay && activityNo == 0)
				{
					html += '<span class="activity-author">' + Strings.encodeTags(activity.author) + '</span>';
					html += '<div class="activity-link">';
					html += '<a href="https://theysaidso.com" title="Powered by quotes from The Said So" target="_blank" rel="noopener noreferrer">theysaidso.com</a>';
					html += '</div>';
				}
				html += '</div>';
				if (activity.quoteOfTheDay && activityNo == 0)
				{
					Doc.element('quote-of-the-day').innerHTML = html;
					html = '';
				}
			}
			Doc.element('activity-log-entries').innerHTML = html;
		}
	}
	
	// Pad a numeric value with leading zeros so that a string with at least 3 digits is created
	index.zeroPad = function(val)
	{
		if (val < 10) return '00'+val;
		if (val < 100) return '0'+val;
		return val;
	}
	
	index.displayCounterAge = function()
	{
		if (counterInfo == null || !counterInfo.countTime) return;
		
		// We have count information
		var formattedDate = Reggie.reformatDate(counterInfo.countTime);

		// Generate a message
		var counterAge = '';
		var time = new Date().getTime();
		counterInfo.infoAgeInSeconds = Math.floor((time - counterInfo.infoTime) / 1000);
		counterInfo.ageInSeconds = counterInfo.countAge + counterInfo.infoAgeInSeconds;
		counterInfo.ageInMinutes = Math.floor(counterInfo.ageInSeconds / 60);
		
		if (counterInfo.ageInSeconds < 60)
		{
			// Counters are less than 1 minute old, display age in seconds
			counterAge += counterInfo.ageInSeconds == 1 ? '1 second ago' : counterInfo.ageInSeconds + ' seconds ago';
		}
		else if (counterInfo.ageInMinutes < 60)
		{
			// Counters are less than 1 hour old, display age in minutes
			counterAge += counterInfo.ageInMinutes == 1 ? '1 minute ago' : counterInfo.ageInMinutes + ' minutes ago';
		}
		else
		{
			// Display age as date and time
			counterAge += formattedDate;
		}
		Doc.element('counter-age').innerHTML = counterAge;
		Doc.element('counter-age').title = formattedDate;
		
		// Show or hide icons
		Doc.hide('counter-loading');
		Doc.hide('counter-refresh');
		if (counterInfo.isCounting) 
		{
			Doc.show('counter-loading', 'inline');
		}
		else if (counterInfo.ageInMinutes >= 5)
		{
			Doc.show('counter-refresh', 'inline');
		}
	}
	
	index.updateCounterAge = function()
	{
		var nextUpdate = 15; // 15 seconds until next update
		var reloadCounts = false; // By default, no reload of counts
		
		if (counterInfo == null)
		{
			// We have no info yet, try again in 1 second...
			nextUpdate = 1;
		}
		else
		{
			if (counterInfo.countTime)
			{
				index.displayCounterAge();
				if (counterInfo.ageInSeconds < 60)
				{
					// Next update should be on even 5 second intervals
					nextUpdate = 5 - counterInfo.ageInSeconds % 5;
				}
				if (counterInfo.infoAgeInSeconds > 300)
				{
					// If our information is more than 5 minutes old, request an update from the server
					reloadCounts = true;
				}
			}
			
			// If a count is in progress request an update from the server
			if (counterInfo.isCounting)
			{
				reloadCounts = true;
				nextUpdate = 2;
			}
			
			// If the service is not running, we need an update from the server
			if (!counterInfo.isRunning)
			{
				reloadCounts = true;
			}
		}
			
		if (reloadCounts) index.loadCounts();
		setTimeout(index.updateCounterAge, nextUpdate * 1000);
	}
	
	index.caseSummary = function()
	{
		var frm = document.forms['reggie'];
		
		var caseName = frm.caseName.value;
		if (!caseName)
		{
			Forms.showNotification('caseName', 'Please enter a 7-digit case id', null, null, 'left');
			return;
		}
		var url = 'reports/case_summary.jsp?ID='+App.getSessionId();
		url += '&caseName='+encodeURIComponent(caseName);
		url += '&highlight='+encodeURIComponent(caseName);
		location.href = url;
	}
	
	index.testSnake = function(event)
	{
		if (event.currentTarget.value == 'Snak' && event.charCode == 101)
		{
			setTimeout(index.showSnake, 500);
		}
	}

	index.showSnake = function()
	{
		var msg = '<img src="images/snake-icon-64.png" style="float: right;">'
		msg += '– There\'s a big snake in the plane, Jock!<br>';
		msg += '– Oh, that\'s just my pet snake, Reggie!<br>';
		msg += '– I hate snakes, Jock! I hate \'em!<br>';
		msg += '– Come on! Show a little backbone, will ya?<br>';
		msg += '<div class="snake-quote-source">Indiana Jones - Raiders of the Lost Ark</div>';
		Forms.showNotification('caseName', msg, 'snake-quote', 'pointer-above');
		var url = 'Session.servlet?ID='+App.getSessionId();
		url += '&cmd=ReggieTheSnakeWasSummoned';
		Wizard.asyncJsonRequest(url);
	}
	
	return index;
}();

Doc.onLoad(Index.initPage);

