var DnaNormalize = function()
{
	var normalize = {};

	// The minial volume of DNA we want to take when normalizing
	// For DNA with high concentration we must make a larger mix to reach the target concentation
	var MINIMAL_DNA_VOLUME = 1; // µl
	var hasInitProtocols = false;
	var protocolIsValid = false;
	
	var selectedDna;
	var subtypeDna = null;
	
	var newDna = [];
	
	// Page initialization
	normalize.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', normalize.validateStep1);
		Buttons.addClickHandler('btnSelectDna', normalize.selectDna);
		Events.addEventHandler('dna', 'base-selected', normalize.onDnaSelected);
		
		// Step 2
		Events.addEventHandler('normalizationProtocol', 'change', normalize.protocolOnChange);
		Events.addEventHandler('normalizationProtocol', 'keyup', normalize.protocolOnChange);
		Events.addEventHandler('normalizedPlate', 'change', normalize.normalizedPlateOnChange);
		Events.addEventHandler('normalizedPlate', 'keyup', normalize.normalizedPlateOnChange);
		Events.addEventHandler('step-2', 'wizard-initialize', normalize.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', normalize.validateStep2);

		// Step 3
		Events.addEventHandler('step-3', 'wizard-initialize', normalize.initializeStep3);
		Events.addEventHandler('normalizationDate', 'blur', Wizard.validateDate);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goprint', normalize.printVersion);
		Buttons.addClickHandler('goregister', Wizard.goRegister);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', normalize.submit);

		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetDNAToNormalize';
		Wizard.showLoadingAnimation('Loading DNA extracts...');
		Wizard.asyncJsonRequest(url, normalize.onDnaLoaded);
	}
	
	normalize.onDnaLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var dnaExtracts = response.dnaExtracts;
		
		dnaExtracts.sort(normalize.sortByQiacubePosition);
		
		if (dnaExtracts.length > 0)
		{
			var qiacubeRun = dnaExtracts[0].QiacubeDate + '#' + dnaExtracts[0].QiacubeRunNo;
			for (var i=0; i < dnaExtracts.length; i++)
			{
				var dna = dnaExtracts[i];
				var selected = (dna.QiacubeDate + '#' + dna.QiacubeRunNo) == qiacubeRun;
				normalize.addDnaOption(frm.dna, dna, selected);
			}
			Wizard.setInputStatus('dna', 'valid');
		}
		else
		{	
			Wizard.setInputStatus('dna', 'invalid', 'No DNA available for normalization.');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	/**
		Sort by qiacube date, run number and position
	*/
	normalize.sortByQiacubePosition = function(a, b)
	{
		if (a.QiacubeDate == b.QiacubeDate)
		{
			if (a.QiacubeRunNo == b.QiacubeRunNo)
			{
				return a.QiacubePosition-b.QiacubePosition;
			}
			else
			{
				return a.QiacubeRunNo - b.QiacubeRunNo;
			}
		}
		else
		{
			return a.QiacubeDate < b.QiacubeDate ? -1 : 1;
		}

		return a.id - b.id;
	}


	/**
		Sort by qiacube date, run number and position
	*/
	normalize.sortByStorageLocation = function(a, b)
	{
		if (a.bioWell && b.bioWell)
		{
			if (a.bioWell.bioPlate.name == b.bioWell.bioPlate.name)
			{
				return a.bioWell.location < b.bioWell.location ? -1 : 1;
			}
			else
			{
				return a.bioWell.bioPlate.name < b.bioWell.bioPlate.name ? -1 : 1;
			}
		}	
		return a.id - b.id;
	}
	
	/**
		Sort wells by column and row
	 */
	normalize.sortWellsByColumnAndRow = function(a, b)
	{
		if (a.column == b.column)
		{
			return a.row - b.row;
		}
		else
		{
			return a.column - b.column;
		}
	}
	
	normalize.selectDna = function()
	{
		var frm = document.forms['reggie'];
		if (frm.dna.disabled) return;
		
		if (subtypeDna == null)
		{
			subtypeDna = Reggie.getSubtypeInfo('DNA');
		}
		
		newDna = [];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeDna.id;
		Dialogs.selectItem('EXTRACT', 'dna', 1, url);
	}

	normalize.onDnaSelected = function(event)
	{
		var dna = event.detail;
		
		var frm = document.forms['reggie'];
		var dnaList = frm.dna;
		var isNew = true;
		for (var i = 0; i < dnaList.length; i++)
		{
			if (dnaList[i].value == dna.id)
			{
				dnaList[i].selected = true;
				isNew = false;
			}
		}
		
		if (isNew) newDna[newDna.length] = dna.id;
		
		if (event.detail.remaining == 0 && newDna.length > 0)
		{
			// Get more information about the selected dna
			var url = '../Extraction.servlet?ID='+App.getSessionId();
			url += '&cmd=GetDNAInfoForNormalize&dnaIds=' + newDna.join(',');
			Wizard.asyncJsonRequest(url, normalize.onDnaInfoLoaded);
		}
	}

	normalize.onDnaInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var dnaExtracts = response.dnaExtracts;
		dnaExtracts.sort(normalize.sortByQiacubePosition);
		
		for (var dnaNo = 0; dnaNo < dnaExtracts.length; dnaNo++)
		{
			var dna = dnaExtracts[dnaNo];
			normalize.addDnaOption(frm.dna, dna, true);
		}
		Wizard.setInputStatus('dna', 'valid');
	}
	
	normalize.addDnaOption = function(list, dna, selected)
	{
		var name = dna.name;
		if (dna.bioWell)
		{
			name += ' -- ' + Reggie.reformatDate(dna.QiacubeDate) + '; Run ' + dna.QiacubeRunNo + '; Pos ' + dna.QiacubePosition;
		}

		var option = new Option(name, dna.id, selected, selected);
		if (dna.specimen && dna.specimen.YellowLabel != null) option.className = 'yellow';
		option.dna = dna;
		list[list.length] = option;
	}
	
	normalize.validateStep1 = function(event)
	{
		var frm = document.forms['reggie'];
		var dnaList = frm.dna;
		
		selectedDna = [];
		for (var i=0; i < dnaList.length; i++)
		{
			if (dnaList[i].selected)
			{
				selectedDna[selectedDna.length] = dnaList[i].dna;
			}
		}
			
		if (selectedDna.length == 0)
		{
			Wizard.setInputStatus('dna', 'invalid', 'Not selected', 'invalid');
			event.preventDefault();
		}
		else
		{
			Wizard.setInputStatus('dna', 'valid');
		}
	}


	normalize.initializeStep2 = function()
	{
		normalize.buildProtocol();
		
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetActivePreNormalizedDNAPlates&numFreeWells=' + selectedDna.length;
		Wizard.asyncJsonRequest(url, normalize.onPlateInfoLoaded);
	}
	
	normalize.onPlateInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var plates = response.plates;
		for (var plateNo = 0; plateNo < plates.length; plateNo++)
		{
			var plate = plates[plateNo];
			var title = '';
			if (plate.id == 0) 
			{
				title = 'Create: ' + plate.name;
			}
			else
			{
				title = plate.name + ' -- ' + plate.freeWells + ' free wells';
			}
			var option = new Option(title, plate.id);
			plate.bioWells.sort(normalize.sortWellsByColumnAndRow);
			option.plate = plate;
			frm.normalizedPlate[frm.normalizedPlate.length] = option;
		}
		Wizard.setInputStatus('normalizedPlate', 'valid');

		Wizard.setCurrentStep(2);
		Doc.show('gonext');
		Doc.show('goprint');
		Doc.show('gocancel');
		Wizard.keepSessionAlive();
		Reggie.loadProtocols('DNA_NORMALIZATION_PROTOCOL', 'normalizationProtocol', 'DNA_TARGET_AMOUNT,DNA_MINIMAL_AMOUNT,DNA_TARGET_VOLUME,MIPS_PANEL');
		frm.normalizationProtocol.focus();
	}
	
	normalize.buildProtocol = function()
	{
		var frm = document.forms['reggie'];
		// Check if all DNA from same Qiacube run
		var qiacubeRun = selectedDna[0].QiacubeDate + '#' + selectedDna[0].QiacubeRunNo;
		var sameQiacubeRun = true;
		for (var i=1; i < selectedDna.length; i++)
		{
			var dna = selectedDna[i];
			if (qiacubeRun != dna.QiacubeDate+'#'+dna.QiacubeRunNo)
			{
				sameQiacubeRun = false;
				break;
			}
		}
		
		if (sameQiacubeRun)
		{
			selectedDna.sort(normalize.sortByQiacubePosition);
			Doc.show('qiacube-info');
			Doc.element('qiacube-date').innerHTML = Reggie.reformatDate(selectedDna[0].QiacubeDate);
			Doc.element('qiacube-run').innerHTML = selectedDna[0].QiacubeRunNo;
		}
		else
		{
			selectedDna.sort(normalize.sortByStorageLocation);
		}
		
		var html = '<table id="extractTable">';
		html += '<thead class="bg-filled-100">';
		// First header row
		html += '<tr>';
		html += '<th>DNA</th>';
		if (sameQiacubeRun)
		{
			html += '<th>Qiacube</th>';
		}
		html += '<th>Storage</th>';
		html += '<th>Conc.</th>';
		html += '<th>Remain</th>';	
		html += '<th colspan="2">Volumes</th>';
		html += '<th>Normalized</th>';
		html += '<th></th>';
		html += '</tr>';
		// Second header row
		html += '<tr>';
		html += '<th></th>';
		if (sameQiacubeRun)
		{
			html += '<th>position</th>';
		}
		html += '<th>box</th>';
		html += '<th>(ng/µl)</th>';
		html += '<th>(µg)</th>';
		html += '<th>(µl)</th>';
		html += '<th>H<sub>2</sub>O</th>';
		html += '<th>plate</th>';
		html += '<th>Remarks</th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		
		var expectedQiacubePosition = 0;
		var homeUrl = Data.get('page-data', 'home-url');
		var yellowImg = '<img src="'+homeUrl+'/images/yellow-label.png">';
		for (var i=0; i < selectedDna.length; i++)
		{
			var dna = selectedDna[i];
			var isYellow = dna.specimen && dna.specimen.YellowLabel != null;
			if (sameQiacubeRun)
			{
				expectedQiacubePosition++;
				while (expectedQiacubePosition != dna.QiacubePosition && expectedQiacubePosition <= 12)
				{
					html += normalize.insertEmpty(expectedQiacubePosition);
					expectedQiacubePosition++;
				}
			}

			var img = isYellow ? yellowImg : '';
			html += '<tr class="highlight'+(isYellow ? ' yellow-specimen' : '')+'" id="dnaRow.'+dna.id+'">';
			html += '<td class="extract if-yellow">'+img+Strings.encodeTags(dna.nextAliquotName)+'</td>';
			if (sameQiacubeRun)
			{
				html += '<td class="qiacubepos">'+dna.QiacubePosition+'</td>';
			}
			html += '<td class="box">'+(dna.bioWell?Strings.encodeTags(dna.bioWell.bioPlate.name + ' ' + dna.bioWell.location):'unknown')+'</td>';
			html += '<td class="conc">'+ Numbers.formatNumber(dna.conc, 1) + '</td>';
			html += '<td class="remain">'+ Numbers.formatNumber(dna.remainingQuantity, 2) + '</td>';
			html += '<td class="volume" id="dnaVolume.'+dna.id+'"></td>';
			html += '<td class="water" id="dnaWater.'+dna.id+'"></td>';
			html += '<td class="plate" id="dnaLocation.'+dna.id+'"></td>';
			html += '<td class="remarks" id="remarks.'+dna.id+'"></td>';
			html += '</tr>'
		}
		if (sameQiacubeRun)
		{
			expectedQiacubePosition++;
			while (expectedQiacubePosition <= 12)
			{
				html += normalize.insertEmpty(expectedQiacubePosition);
				expectedQiacubePosition++;
			}
		}
		html += '</tbody>';
		html += '</table>';
		
		Doc.element('dnaNormalize').innerHTML = html;
	}
	
	normalize.normalizedPlateOnChange = function()
	{
		var frm = document.forms['reggie'];
		Doc.element('normalized-plate-name').innerHTML =  Strings.encodeTags(frm.normalizedPlate[frm.normalizedPlate.selectedIndex].text);
		
		Wizard.showGoNextConfirmation(true, 'Print the lab protocol before continuing with the registration!');

		var plate = frm.normalizedPlate[frm.normalizedPlate.selectedIndex].plate;
		var wells = plate.bioWells;
		
		var wellNo = 0;
		for (var i=0; i < selectedDna.length; i++)
		{
			var dna = selectedDna[i];
			if (!dna.notEnoughQuantity)
			{
				while (!wells[wellNo].canAdd) wellNo++;
				Doc.element('dnaLocation.'+dna.id).innerHTML = wells[wellNo].location;
				dna.well = wells[wellNo];
				wellNo++;
			}
			else
			{
				Doc.element('dnaLocation.'+dna.id).innerHTML = '';
				dna.well = null;
			}
		}
	}
	
	normalize.protocolOnChange = function()
	{
		var frm = document.forms['reggie'];

		if (!hasInitProtocols)
		{
			hasInitProtocols = true;
			if (frm.normalizationProtocol.length == 0)
			{
				Wizard.setFatalError('No DNA normalization protocols found. Please ask the server administrator to create at least one protocol.');
				return;
			}
			for (var i=0; i < frm.normalizationProtocol.length; i++)
			{
				var p = frm.normalizationProtocol[i].item;
				
				var unit = 'µg';
				var factor = 1;
				if (p.DNATargetAmount <= 0.5)
				{
					unit = 'ng';
					factor = 1000;
				}
				
				var text = p.name + ' (';
				text += (p.MIPS_Panel ? Strings.encodeTags(p.MIPS_Panel) : 'NO MIPS_Panel!') + '; ';
				if (p.DNAMinimalAmount) text += Reggie.formatNumber(p.DNAMinimalAmount * factor, '', 1) + '-';
				text += p.DNATargetAmount ? Reggie.formatNumber(p.DNATargetAmount * factor, '', 1) : '? ';
				text += unit;
				text += ' in '+(p.DNATargetVolume || '? ')+'µl)';
				frm.normalizationProtocol[i].text = text;
			}
		}

		Wizard.setInputStatus('normalizationProtocol');
		protocolIsValid = true;
		Wizard.showGoNextConfirmation(true, 'Print the lab protocol before continuing with the registration!');

		var protocol = frm.normalizationProtocol[frm.normalizationProtocol.selectedIndex].item;
		if (!protocol.MIPS_Panel)
		{
			Wizard.setInputStatus('normalizationProtocol', 'invalid',  'No MIPS_Panel!');
			protocolIsValid = false;
		}
		if (!protocol.DNATargetAmount || !protocol.DNATargetVolume)
		{
			protocol = {};
			protocol.DNATargetAmount = 0.5;
			protocol.DNATargetVolume = 10;
			protocolIsValid = true;
			
			if (protocolIsValid)
			{
				var msg = 'Missing normalization data; assuming ';
				msg += protocol.DNATargetAmount+ 'µg in '+protocol.DNATargetVolume+'µl.';
				Wizard.setInputStatus('normalizationProtocol', 'warning',  msg);
			}
		}
		else if (protocolIsValid)
		{
			Wizard.setInputStatus('normalizationProtocol', 'valid');
		}
		if (!protocol.DNAMinimalAmount) protocol.DNAMinimalAmount = protocol.DNATargetAmount;
		Doc.element('normalization-protocol').innerHTML =  Strings.encodeTags(frm.normalizationProtocol[frm.normalizationProtocol.selectedIndex].text);

		// The concentration of the normalized DNA
		var normalizedConc = protocol.DNATargetAmount / protocol.DNATargetVolume; // µg/µl
		
		for (var i=0; i < selectedDna.length; i++)
		{
			var dna = selectedDna[i];
			
			// Aim for DNATargetAmount but not more than remiaingQuantity or less than DNAMinimalAmount
			dna.usedQuantity = Math.max(Math.min(dna.remainingQuantity, protocol.DNATargetAmount), protocol.DNAMinimalAmount); // µg
			dna.volume = Math.ceil(10000*dna.usedQuantity/dna.conc) / 10;
			dna.water = protocol.DNATargetVolume-dna.volume; // µl
			
			var remarks = [];
			if (dna.volume < MINIMAL_DNA_VOLUME)
			{
				// Large mix since we do not want to take less than 1µl
				dna.volume = MINIMAL_DNA_VOLUME;
				dna.usedQuantity = dna.volume * dna.conc / 1000; // µg
				var totalVolume = dna.usedQuantity / normalizedConc; // µl
				dna.water = totalVolume - dna.volume;
				remarks[remarks.length] = 'Large mix';
			}
			else if (dna.volume > protocol.DNATargetVolume)
			{
				// The DNA has too low concentration so we can't reach the target amount
				// with the target volume. The protocol may allow us to use a lesser amount
				// so we try this
				dna.volume = protocol.DNATargetVolume;
				dna.water = 0;
				dna.usedQuantity = dna.volume * dna.conc / 1000;
			}
			
			dna.notEnoughQuantity = dna.remainingQuantity < dna.usedQuantity || dna.usedQuantity < protocol.DNAMinimalAmount;
			if (dna.notEnoughQuantity) 
			{
				remarks[remarks.length] = '<span class="flag">Not enough quantity</span>';
			}
			else if (dna.usedQuantity < protocol.DNATargetAmount)
			{
				remarks[remarks.length] = Numbers.formatNumber(dna.usedQuantity * 1000, 0) + 'ng';
			}
			
			Doc.element('dnaVolume.'+dna.id).innerHTML = dna.notEnoughQuantity ? '' : Numbers.formatNumber(dna.volume, 1);
			Doc.element('dnaWater.'+dna.id).innerHTML = dna.notEnoughQuantity ? '' : (dna.water > 0 ? Numbers.formatNumber(dna.water, 1) : '—');
			Doc.element('remarks.'+dna.id).innerHTML = remarks.join(', ');
			Doc.addOrRemoveClass('dnaRow.'+dna.id, 'not-enough-quantity', dna.notEnoughQuantity);
		}
		normalize.normalizedPlateOnChange();
	}
	
	normalize.insertEmpty = function(qiacubePosition)
	{
		var html = '<tr class="highlight empty">';
		html += '<td class="extract"></td>';
		html += '<td class="qiacubepos">'+qiacubePosition+'</td>';
		html += '<td class="box"></td>';
		html += '<td class="conc"></td>';
		html += '<td class="remain"></td>';
		html += '<td class="volume"></td>';
		html += '<td class="water"></td>';
		html += '<td class="plate"></td>';
		html += '<td class="remarks">Not selected for pre-normalization</td>';
		html += '</tr>'
		return html;
	}
	
	normalize.printVersion = function()
	{
		Wizard.hideGoNextConfirmation();
		Doc.removeClass('gonext', 'disabled');
		var frm = document.forms['reggie'];
		var reportName = 'DNA Normalization Protocol';
		var printNote = '<b>Note!</b> For better printing set page orientation to <i>portrait</i>.';
		Reggie.openPrintWindow('full-protocol', reportName, 'portrait', printNote, '../', 'normalize_extract.css');
	}

	normalize.validateStep2 = function(event)
	{
		var valid = protocolIsValid;
		if (!valid) event.preventDefault();
	}

	normalize.initializeStep3 = function()
	{
		var frm = document.forms['reggie'];
		Wizard.setCurrentStep(3);
		Doc.show('goregister');
		Doc.show('gocancel');
		Events.sendChangeEvent('normalizationDate');
		frm.normalizationDate.focus();
	}
	
	normalize.submit = function()
	{
		var frm = document.forms['reggie'];
		
		var submitInfo = {};
		submitInfo.normalizationProtocol = parseInt(frm.normalizationProtocol.value, 10);
		submitInfo.normalizationDate = frm.normalizationDate.value;
		submitInfo.normalizationOperator = frm.normalizationOperator.value;
		submitInfo.normalizedPlate = parseInt(frm.normalizedPlate.value, 10);
		
		var dnaAliquots = [];
		for (var i=0; i < selectedDna.length; i++)
		{
			var dna = selectedDna[i];
			if (!dna.notEnoughQuantity)
			{
				var normalizedDna = {};
				normalizedDna.parentId = dna.id;
				normalizedDna.name = dna.nextAliquotName;

				normalizedDna.usedQuantity = dna.usedQuantity; // µg
				normalizedDna.conc = 1000 * dna.usedQuantity / (dna.volume+dna.water); // ng/µl
				normalizedDna.ConcType = dna.NDConc ? 'ND_CONC' : 'QUBIT_CONC';
				
				normalizedDna.row = dna.well.row;
				normalizedDna.column = dna.well.column;
				
				dnaAliquots[dnaAliquots.length] = normalizedDna;
			}
			else
			{
				var flaggedDna = {};
				flaggedDna.id = dna.id;
				flaggedDna.flag = 'NotEnoughRemainingQuantity';
				dnaAliquots[dnaAliquots.length] = flaggedDna;
			}
		}
		
		submitInfo.dnaAliquots = dnaAliquots;

		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterNormalizedDNA';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, normalize.submissionResults, 'POST', JSON.stringify(submitInfo));
	}

	normalize.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return normalize;
}();

Doc.onLoad(DnaNormalize.initPage);

